#include <stdio.h>
#include <emscripten.h> // note we added the emscripten header
#include <iostream>
#include <exception>
#include <sstream>
#include "../etvo.h"
#include "parserEmscripten.h"

using namespace std;
using namespace etvo;


// C fonction to use ETVO with Emscripten

/* Compilation with Emscripten on LUbuntu
 Terminal : go to emsdk/upstream/emscripten directory  (./em++ compiler)
 Put the etvolib in a subdirectory 
	emsdk/upstream/emscripten/bc/etvolib [the whole ETVO lib as given on the web]
				with etvo.cpp, parsersEtvo.cpp ... etvoEM.cpp parserEmscripten.h and parserEmscripen.cpp
*/
// Compilation command
/*
./em++ -s USE_BOOST_HEADERS=1   // because of boost lib
	-s EXIT_RUNTIME=0  	// to keep function loaded after the end of main()
	-s EXTRA_EXPORTED_RUNTIME_METHODS='["cwrap"]'  // so that the C-function can be Wrapped in Javascript
	bc/etvolib/etvo.cpp    // 1st source (etvo core)
	bc/etvolib/parserEmscripten.cpp //2nd source = ETVO parsers + specifi emscripten parser (a kind of wrapper to existing parsers)
	bc/etvolib/etvoEM.cpp   // 3rd soure = this file with the C-function   char * parse(char *)
	-o bc/etvoEM.js    // the outpput is  etvoEM.wasm + etvoEM.js (javascript glue)
*/


std::string globalSTR;   //IMPORTANT : this global variable is necessary to be able to return a pointer on a char array

extern "C"
{
// This function parse a string with the ETVO calculator and gives a result as a string too
const char * EMSCRIPTEN_KEEPALIVE parse(char * str)   // parse is the only C-function used in Javascript
{
     string S=str;
     stringstream buffer;
     std::streambuf * old = std::cout.rdbuf(buffer.rdbuf());    // cout stream redirected unto string stream 
     calculatorEtvo::parseEM(S);  // parser function defined in parserEmscripten.h
     cout << endl;
     globalSTR=buffer.str();  // retrieves the cout string
     return globalSTR.c_str();    // OK because globalSTR is a global variable
}

const char * EMSCRIPTEN_KEEPALIVE getVars() 
{
	std::string varNames="";
	for(auto el:calculatorEtvo::vars_MM) varNames+=el.first +",";
	for(auto el:calculatorEtvo::vars_Ed) varNames+=el.first +",";
	for(auto el:calculatorEtvo::vars_Tg) varNames+=el.first +",";
	for(auto el:calculatorEtvo::vars_mMM) varNames+=el.first +",";
	for(auto el:calculatorEtvo::vars_mEd) varNames+=el.first +",";
	for(auto el:calculatorEtvo::vars_mTg) varNames+=el.first +",";
	globalSTR=varNames;
	return globalSTR.c_str();
}
}

int main()
{
	cout << "ETVO Web" << endl;	
	cout << endl;
}
