//mmgdW.cpp 

#include<iostream>
#include<string>
#include<vector>
#include<sstream>
#include<emscripten.h>
#include <emscripten/bind.h>
#include "../etvo.h"
#include "../parsers/parser.h"

///////////////////////////////////
//
// COMPILATION Emscripten
//
//  	em++ -lembind mmgdW.cpp ../etvo.cpp ../parsers/parser.cpp ../parsers/parserSeries.cpp -o mmgdW.js -s WASM=1 -s EXIT_RUNTIME=0
//
///////////////////////////////////

using namespace etvo;
using namespace std;
using namespace emscripten;

/* ASSUMPTION = in the javascript part, series and smatrix are assumed 
to be defined as 
  // var s= new series([0,2],[1,3,3,4],3,5)
function series(p,q,nu,tau)
{ 
	this.p=p;     // p is assumed to be an array [n0,t0,n1,t1,...] 
	this.q=q;	  // q is assumed to be an array [N0,T0,N1,T1,...] 
	this.nu=nu;   // nu, tau are two integers
	this.tau=tau;	
	...
}

function smatrix(mat)    
{
	this.mat=mat;  // mat is assumed to be an array of arrays of series
	
}
*/

// JS object series (emscripten::val) -> etvo::series
series valToSeries(emscripten::val s1)
{
  vector<int> vp1=vecFromJSArray<int>(s1["p"]);     // JS array -> vector<int>
  vector<int> vq1=vecFromJSArray<int>(s1["q"]);    
  int nu1=s1["nu"].as<int>();		// s1.nu -> int
  int tau1=s1["tau"].as<int>();  	// s1.tau -> int  
  poly p1,q1;  
  for(int i=0;i<vp1.size();i+=2){  p1=p1 + gd(vp1[i],vp1[i+1]); }  
  for(int i=0;i<vq1.size();i+=2){  q1=q1 + gd(vq1[i],vq1[i+1]); }    
  gd r(nu1,tau1);
  series se1(p1,q1,r);  // create a series (etvo::seris object)
  se1.canon();
  return se1;
}

// etvo::series -> val (JS object series)
emscripten::val seriesToVal(series sRes)
{  
  vector<int> rp,rq;
  for(int i=0;i<sRes.getP().size();i++){
	rp.push_back(sRes.getP()[i].getg());
	rp.push_back(sRes.getP()[i].getd());
  }	
  for(int i=0;i<sRes.getQ().size();i++){
	rq.push_back(sRes.getQ()[i].getg());
	rq.push_back(sRes.getQ()[i].getd());
  } 
  val arrayP=val::array(rp.begin(),rp.end());
  val arrayQ=val::array(rq.begin(),rq.end());		    
  val S=val::global("series").new_(arrayP,arrayQ,sRes.getR().getg(),sRes.getR().getd());
  return S;	  
}


// EXPOSED functions to  compute oplus otimes and kleene star

 emscripten::val sparse(std::string str)
 {     
   return seriesToVal(etvo::parser::parseSeries(str));
 }


 emscripten::val splus(emscripten::val s1,emscripten::val s2) {     
    
  series se1=valToSeries(s1);
  series se2=valToSeries(s2);  
  series sRes=se1+se2;  
  return seriesToVal(sRes);
 }
 
 emscripten::val s_inf(emscripten::val s1,emscripten::val s2) {         
  series se1=valToSeries(s1);
  series se2=valToSeries(s2);  
  series sRes=se1.inf(se2);  
  return seriesToVal(sRes);
 }
 
 emscripten::val sfrac(emscripten::val s1,emscripten::val s2) {     
    
  series se1=valToSeries(s1);
  series se2=valToSeries(s2);  
  series sRes=se1.frac(se2);  
  return seriesToVal(sRes);
 }
 
 emscripten::val stimes(emscripten::val s1,emscripten::val s2) {          
  series se1=valToSeries(s1);
  series se2=valToSeries(s2);  
  series sRes=se1*se2;  
  return seriesToVal(sRes);
 }
 
  
 emscripten::val skleene(emscripten::val s1) {     
 series se1=valToSeries(s1);  
  series sRes=se1.star();  
  return seriesToVal(sRes);
 }
  
  emscripten::val scanon(emscripten::val s1) {     
  series se1=valToSeries(s1);  
  se1.canon();  
  return seriesToVal(se1);
 }
 
  emscripten::val sprcaus(emscripten::val s1) {     
  series se1=valToSeries(s1);  
  se1=se1.prcaus();
  return seriesToVal(se1);
 }
 
 
 std::string seriestring(emscripten::val s1) {     
  series se1=valToSeries(s1); 
  stringstream ss;
  ss << se1;
  return ss.str();
 }
 
 std::string sToString(emscripten::val s1) {     
  series se1=valToSeries(s1); 
  poly p=se1.getP();
  poly q=se1.getQ();  
  stringstream ss;
  ss << "sseries([";
  if(!p.isEpsilon())
  {
	 for(int i=0;i<p.size();i++)
	 {
		 ss << p[i].getg() <<"," << p[i].getd();
		 if(i!=p.size()-1) ss <<",";
	 }
  }
  ss << "],[";
  
  if(!q.isEpsilon())
  {
	 for(int i=0;i<q.size();i++)
	 {
		 ss << q[i].getg() <<"," << q[i].getd();
		 if(i!=q.size()-1) ss <<",";
	 }
  }
  ss << "]," << se1.getR().getg() << "," << se1.getR().getd()<<")";  
  return ss.str();
 }
 
 

/////////////////////////////////////////////////////////////////////////
matrix<series> valToSmatrix(emscripten::val smat1,int r,int c)
{
  val data=smat1["mat"];    
  std::vector<val> rows=vecFromJSArray<val>(data);
  matrix<series> MAT(r,c);
  int l=rows.size(); 
  for(int i=0;i<l;i++)
  {
	std::vector<val> ri=vecFromJSArray<val>(rows[i]);
	int c= ri.size();
	for(int j=0;j<c;j++)
    {  
	 MAT(i,j)= valToSeries(ri[j]);  
	}
  }   
  return MAT;
}


emscripten::val smatrixToVal(matrix<series> M)
{
	series eps;
	vector<val> rows;
	for(int i=0;i<M.GetRow();i++)
	{
		vector<val> v(M.GetColumn(),seriesToVal(eps));
		for(int j=0;j<M.GetColumn();j++)
		{
			v[j]=seriesToVal(M(i,j));			
		}
		val rowi=val::array(v.begin(),v.end());
		rows.push_back(rowi);
	}
	val mat=val::array(rows.begin(),rows.end());
	val Mres=val::global("smatrix").new_(mat);
	return Mres;
}

// Exported smatrix operations
emscripten::val smplus(emscripten::val sm1,emscripten::val sm2,int r,int c) {  
  matrix<series> m1=valToSmatrix(sm1,r,c);
  matrix<series> m2=valToSmatrix(sm2,r,c);   
  matrix<series> mres(1,1);
  mres=m1+m2;  
  return smatrixToVal(mres);
 }

emscripten::val smtimes(emscripten::val sm1,emscripten::val sm2,int r1,int c1,int c2) {  
  matrix<series> m1=valToSmatrix(sm1,r1,c1);
  matrix<series> m2=valToSmatrix(sm2,c1,c2);   
  matrix<series> mres(1,1);
  mres=m1*m2;  
  return smatrixToVal(mres);
 }

emscripten::val sminf(emscripten::val sm1,emscripten::val sm2,int r,int c) {  
  matrix<series> m1=valToSmatrix(sm1,r,c);
  matrix<series> m2=valToSmatrix(sm2,r,c);   
  matrix<series> mres(1,1);
  mres=m1.inf(m2);  
  return smatrixToVal(mres);
 }
 
 emscripten::val smlfrac(emscripten::val sm1,emscripten::val sm2,int r1,int c1,int r2,int c2) {  
  matrix<series> m1=valToSmatrix(sm1,r1,c1);
  matrix<series> m2=valToSmatrix(sm2,r2,c2);   
  matrix<series> mres(1,1);
  mres=m1.lfrac(m2);  
  return smatrixToVal(mres);
 }
 
 emscripten::val smrfrac(emscripten::val sm1,emscripten::val sm2,int r1,int c1,int r2,int c2) {  
  matrix<series> m1=valToSmatrix(sm1,r1,c1);
  matrix<series> m2=valToSmatrix(sm2,r2,c2);   
  matrix<series> mres(1,1);
  mres=m1.rfrac(m2);  
  return smatrixToVal(mres);
 }

emscripten::val smkleene(emscripten::val sm1,int r1,int c1) {  
  matrix<series> m1=valToSmatrix(sm1,r1,c1);
  matrix<series> mres(1,1);
  mres=m1.star();  
  return smatrixToVal(mres);
 }

 std::string smatrixstring(emscripten::val sm1,int r1,int c1) {     
  matrix<series> m1=valToSmatrix(sm1,r1,c1);  
  stringstream ss;
  ss << m1;
  return ss.str();
 }
  
 
EMSCRIPTEN_BINDINGS(module) {		
 //series functions 
  emscripten::function("splus", &splus);     // can be called from JS 
  emscripten::function("sinf", &s_inf);     // can be called from JS 
  emscripten::function("sfrac", &sfrac);     // can be called from JS   
  emscripten::function("stimes", &stimes); 
  emscripten::function("skleene", &skleene);     
  emscripten::function("scanon", &scanon);
  emscripten::function("sprcaus", &sprcaus);  
  emscripten::function("seriestring", &seriestring);     
  emscripten::function("sToString", &sToString);     
  emscripten::function("sparse", &sparse);
  
  
  //smatrix functions
  emscripten::function("smplus", &smplus);     // can be called from JS 
  emscripten::function("smtimes", &smtimes);     // can be called from JS  
  emscripten::function("smkleene", &smkleene);     // can be called from JS  
  emscripten::function("smatrixstring", &smatrixstring);      
  emscripten::function("sminf", &sminf);  
  emscripten::function("smlfrac", &smlfrac);  
  emscripten::function("smrfrac", &smrfrac);  
}

int main()
{
	EM_ASM(	console.log("mmgdWasm.wasm loaded ...");	);
	return 0;	
}