//mmgdW.cpp 

#include<iostream>
#include<string>
#include<vector>
#include<sstream>
#include<emscripten.h>
#include <emscripten/bind.h>
#include "../etvo.h"
#include "../parsers/parser.h"

///////////////////////////////////
//
// COMPILATION Emscripten
//
//  	em++ -lembind seriesEdW.cpp ../etvo.cpp ../parsers/parser.cpp ../parsers/parserSeriesEd.cpp -o seriesEdW.js -s WASM=1 -s EXIT_RUNTIME=0
//
///////////////////////////////////

using namespace etvo;
using namespace std;
using namespace emscripten;

/* ASSUMPTION = in the javascript part, series and smatrix are assumed 
to be defined as 
  // var s= new seriesEd([...],[...],3,5)
function seriesEd(p,q,nu,tau)
{ 
	this.p=p;     // p is assumed to be an array [n0,m0,b0,np0,t0,n1,m1,b1,np1,t1,...] 
	this.q=q;	  // q is assumed to be an array [N0,M0,B0,NP0,T0,...] 
	this.nu=nu;   // nu, tau are two integers
	this.tau=tau;	
	...
}

function smatrix(mat)    
{
	this.mat=mat;  // mat is assumed to be an array of arrays of series
	
}
*/

// JS object series (emscripten::val) -> etvo::seriesEd
seriesEd valToSeriesEd(emscripten::val s1)
{
  vector<int> vp1=vecFromJSArray<int>(s1["p"]);     // JS array -> vector<int>
  vector<int> vq1=vecFromJSArray<int>(s1["q"]);    
  int nu1=s1["nu"].as<int>();		// s1.nu -> int
  int tau1=s1["tau"].as<int>();  	// s1.tau -> int  
  polyEd p1,q1;  
  for(int i=0;i<vp1.size();i+=5){  p1=p1 + Ed(vp1[i],(unsigned)vp1[i+1],(unsigned)vp1[i+2],vp1[i+3],vp1[i+4]); }  
  for(int i=0;i<vq1.size();i+=5){  q1=q1 + Ed(vq1[i],(unsigned)vq1[i+1],(unsigned)vq1[i+2],vq1[i+3],vq1[i+4]); }    
  gd r(nu1,tau1);
  seriesEd se1(p1,q1,r);  // create a seriesEd (etvo::seriesEd object)
  se1.canon();
  return se1;
}



// etvo::seriesEd -> val (JS object series)
emscripten::val seriesEdToVal(seriesEd sRes)
{  
  sRes.toRight();
  vector<int> rp,rq;
  polyEd pRes=sRes.getP();
  Ed m;
  E_op alpha;
  for(int i=0;i<pRes.size();i++){
	m=pRes[i];
	alpha=m.getE_op();	
	for(gNg term:alpha.getTerms())
	{	  
	    term.canonL();
		rp.push_back(term.getNl());
		rp.push_back(term.getM());
		rp.push_back(term.getB());
		rp.push_back(term.getNr());
		rp.push_back(m.getD());				
	}
  }	   
  val arrayP=val::array(rp.begin(),rp.end());
  pRes=sRes.getQ();
  rp.clear();
  for(int i=0;i<pRes.size();i++){
	m=pRes[i];
	alpha=m.getE_op();	
	for(gNg term:alpha.getTerms())
	{	  
		term.canonL();
		rp.push_back(term.getNl());
		rp.push_back(term.getM());
		rp.push_back(term.getB());
		rp.push_back(term.getNr());
		rp.push_back(m.getD());				
	}
  }	     
  val arrayQ=val::array(rp.begin(),rp.end());		    
  val S=val::global("seriesEd").new_(arrayP,arrayQ,sRes.getR().getg(),sRes.getR().getd());
  return S;	  
}

// EXPOSED functions to  compute oplus otimes and kleene star

 emscripten::val sEdparse(std::string str)
 {     
   return seriesEdToVal(etvo::parser::parseSeriesEd(str));
 }
 
 
 std::string seriesEdstring(emscripten::val s1) {     
  seriesEd se1=valToSeriesEd(s1); 
  stringstream ss;
  ss << se1;
  return ss.str();
 }
 
 
 std::string sEdToString(emscripten::val s1) {     
 
 seriesEd se1=valToSeriesEd(s1); 
  stringstream ss;
  ss << se1;
  return ss.str();
  
  /*
  seriesEd se1=valToSeriesEd(s1); 
  polyEd p=se1.getP();
  polyEd q=se1.getQ();  
  stringstream ss;
  ss << "sseriesEd([";
  if(!p.isEpsilon())
  {
	 for(int i=0;i<p.size();i++)
	 {
		 ss << p[i].getg() <<"," << p[i].getd();
		 if(i!=p.size()-1) ss <<",";
	 }
  }
  ss << "],[";
  
  if(!q.isEpsilon())
  {
	 for(int i=0;i<q.size();i++)
	 {
		 ss << q[i].getg() <<"," << q[i].getd();
		 if(i!=q.size()-1) ss <<",";
	 }
  }
  ss << "]," << se1.getR().getg() << "," << se1.getR().getd()<<")";  
  return ss.str();
  */
 }
 


 emscripten::val sEdplus(emscripten::val s1,emscripten::val s2) {     
    
  seriesEd se1=valToSeriesEd(s1);
  seriesEd se2=valToSeriesEd(s2);  
  seriesEd sRes=se1+se2;  
  return seriesEdToVal(sRes);
 }
 
 emscripten::val sEd_inf(emscripten::val s1,emscripten::val s2) {         
  seriesEd se1=valToSeriesEd(s1);
  seriesEd se2=valToSeriesEd(s2);  
  seriesEd sRes=se1.inf(se2);  
  return seriesEdToVal(sRes);
 }
 
 emscripten::val sEdlfrac(emscripten::val s1,emscripten::val s2) {     
    
  seriesEd se1=valToSeriesEd(s1);
  seriesEd se2=valToSeriesEd(s2);  
  seriesEd sRes=se1.lfrac(se2);  
  return seriesEdToVal(sRes);
 }
 
 emscripten::val sEdrfrac(emscripten::val s1,emscripten::val s2) {     
    
  seriesEd se1=valToSeriesEd(s1);
  seriesEd se2=valToSeriesEd(s2);  
  seriesEd sRes=se1.rfrac(se2);  
  return seriesEdToVal(sRes);
 }
 
 emscripten::val sEdtimes(emscripten::val s1,emscripten::val s2) {          
  seriesEd se1=valToSeriesEd(s1);
  seriesEd se2=valToSeriesEd(s2);  
  seriesEd sRes=se1*se2;  
  return seriesEdToVal(sRes);
 }
 
  
 emscripten::val sEdkleene(emscripten::val s1) {     
 seriesEd se1=valToSeriesEd(s1);  
  seriesEd sRes=se1.star();  
  return seriesEdToVal(sRes);
 }
  
  emscripten::val sEdcanon(emscripten::val s1) {     
  seriesEd se1=valToSeriesEd(s1);  
  se1.canon();  
  return seriesEdToVal(se1);
 }
 
  emscripten::val sEdprcaus(emscripten::val s1) {     
  seriesEd se1=valToSeriesEd(s1);  
  se1=seriesEd::toCausal(se1);
  return seriesEdToVal(se1);
 }
 
 
 std::string sEdtoStringRight(emscripten::val s1) {     
  seriesEd se1=valToSeriesEd(s1);  
  se1.toRight();    
  stringstream ss;
  ss << se1;
  return ss.str(); 
 }
 
 std::string sEdtoStringLeft(emscripten::val s1) {     
  seriesEd se1=valToSeriesEd(s1);  
  se1.toLeft();    
  stringstream ss;
  ss << se1;
  return ss.str(); 
 }
 
 

/////////////////////////////////////////////////////////////////////////
matrix<seriesEd> valToSmatrixEd(emscripten::val smat1,int r,int c)
{
  val data=smat1["mat"];    
  std::vector<val> rows=vecFromJSArray<val>(data);
  matrix<seriesEd> MAT(r,c);
  int l=rows.size(); 
  for(int i=0;i<l;i++)
  {
	std::vector<val> ri=vecFromJSArray<val>(rows[i]);
	int c= ri.size();
	for(int j=0;j<c;j++)
    {  
	 MAT(i,j)= valToSeriesEd(ri[j]);  
	}
  }   
  return MAT;
}


emscripten::val smatrixEdToVal(matrix<seriesEd> M)
{
	seriesEd eps;
	vector<val> rows;
	for(int i=0;i<M.GetRow();i++)
	{
		vector<val> v(M.GetColumn(),seriesEdToVal(eps));
		for(int j=0;j<M.GetColumn();j++)
		{
			v[j]=seriesEdToVal(M(i,j));			
		}
		val rowi=val::array(v.begin(),v.end());
		rows.push_back(rowi);
	}
	val mat=val::array(rows.begin(),rows.end());
	val Mres=val::global("smatrixEd").new_(mat);
	return Mres;
}

// Exported smatrix operations
emscripten::val smEdplus(emscripten::val sm1,emscripten::val sm2,int r,int c) {  
  matrix<seriesEd> m1=valToSmatrixEd(sm1,r,c);
  matrix<seriesEd> m2=valToSmatrixEd(sm2,r,c);   
  matrix<seriesEd> mres(1,1);
  mres=m1+m2;  
  return smatrixEdToVal(mres);
 }

emscripten::val smEdtimes(emscripten::val sm1,emscripten::val sm2,int r1,int c1,int c2) {  
  matrix<seriesEd> m1=valToSmatrixEd(sm1,r1,c1);
  matrix<seriesEd> m2=valToSmatrixEd(sm2,c1,c2);   
  matrix<seriesEd> mres(1,1);
  mres=m1*m2;  
  return smatrixEdToVal(mres);
 }

emscripten::val smEdinf(emscripten::val sm1,emscripten::val sm2,int r,int c) {  
  matrix<seriesEd> m1=valToSmatrixEd(sm1,r,c);
  matrix<seriesEd> m2=valToSmatrixEd(sm2,r,c);   
  matrix<seriesEd> mres(1,1);
  mres=m1.inf(m2);  
  return smatrixEdToVal(mres);
 }
 
 emscripten::val smEdlfrac(emscripten::val sm1,emscripten::val sm2,int r1,int c1,int r2,int c2) {  
  matrix<seriesEd> m1=valToSmatrixEd(sm1,r1,c1);
  matrix<seriesEd> m2=valToSmatrixEd(sm2,r2,c2);   
  matrix<seriesEd> mres(1,1);
  mres=m1.lfrac(m2);  
  return smatrixEdToVal(mres);
 }
 
 emscripten::val smEdrfrac(emscripten::val sm1,emscripten::val sm2,int r1,int c1,int r2,int c2) {  
  matrix<seriesEd> m1=valToSmatrixEd(sm1,r1,c1);
  matrix<seriesEd> m2=valToSmatrixEd(sm2,r2,c2);   
  matrix<seriesEd> mres(1,1);
  mres=m1.rfrac(m2);  
  return smatrixEdToVal(mres);
 }

emscripten::val smEdkleene(emscripten::val sm1,int r1,int c1) {  
  matrix<seriesEd> m1=valToSmatrixEd(sm1,r1,c1);
  matrix<seriesEd> mres(1,1);
  mres=m1.star();  
  return smatrixEdToVal(mres);
 }

 std::string smatrixEdstring(emscripten::val sm1,int r1,int c1) {     
  matrix<seriesEd> m1=valToSmatrixEd(sm1,r1,c1);  
  stringstream ss;
  ss << m1;
  return ss.str();
 }
  
 
EMSCRIPTEN_BINDINGS(module) {		
 //seriesEd functions 
 emscripten::function("sEdparse", &sEdparse);
 emscripten::function("seriesEdstring", &seriesEdstring);    
 
  emscripten::function("sEdplus", &sEdplus);     // can be called from JS 
  emscripten::function("sEdinf", &sEd_inf);     // can be called from JS 
  emscripten::function("sEdlfrac", &sEdlfrac);     // can be called from JS   
  emscripten::function("sEdrfrac", &sEdrfrac);     // can be called from JS   
  emscripten::function("sEdtimes", &sEdtimes); 
  emscripten::function("sEdkleene", &sEdkleene);     
  emscripten::function("sEdcanon", &sEdcanon);
  emscripten::function("sEdprcaus", &sEdprcaus);  
   
  emscripten::function("sEdToString", &sEdToString);     
  emscripten::function("sEdtoStringRight", &sEdtoStringRight);     
  emscripten::function("sEdtoStringLeft", &sEdtoStringLeft);     
  
  
  
  //smatrix functions
  emscripten::function("smEdplus", &smEdplus);     // can be called from JS 
  emscripten::function("smEdtimes", &smEdtimes);     // can be called from JS  
  emscripten::function("smEdkleene", &smEdkleene);     // can be called from JS  
  emscripten::function("smatrixEdstring", &smatrixEdstring);      
  emscripten::function("smEdinf", &smEdinf);  
  emscripten::function("smEdlfrac", &smEdlfrac);  
  emscripten::function("smEdrfrac", &smEdrfrac);  
  
}

int main()
{
	EM_ASM(	console.log("seriesEdW.wasm loaded ...");	);
	return 0;	
}