#include "dDd.h"
#include "T_op.h"
#include<sstream>
#include<cmath>
#include<exception>
using namespace std;

namespace etvo
{
/// set the default form as the Left Form
unsigned dDd::_canon=0;  // left = _tr in [_w-1,0]

unsigned dDd::getCanonForm()
{
	return _canon;
}
void dDd::setCanonForm(unsigned val)
{
    _canon=val;
}

/// Create term d^tl.V_m.d^tc.W_b.d^tr
dDd::dDd(int tl, unsigned int v, int tc,unsigned int w, 
				int tr) :_tl(tl), _v(v), _tc(tc),_w(w),_tr(tr)
{
	if (_v == 0) _v = 1;
	if (_w == 0) _w = 1;
	canon();
}

/// Create term d^tl.V_m.d^0.W_b.d^tr
dDd::dDd(int tl, unsigned int v, unsigned int w,
	int tr) :_tl(tl), _v(v), _tc(0), _w(w), _tr(tr)
{
	if (_v == 0) _v = 1;
	if (_w == 0) _w = 1;
	canon();
}


/// Create term d^tl.D_vw.d^tr=d^tl.V_m.d^0.W_m.d^tr
dDd::dDd(int tl, unsigned int vw, 
	int tr):_tl(tl), _v(vw), _tc(0),_w(vw),_tr(tr)
{
	if (_v == 0) _v = 1;
	if (_w == 0) _w = 1;
	canon();
}

/// d^0.V_1.d^tc.W_1.d^0 = d^tc
dDd::dDd(int tc) :_tl(0), _v(1), _tc(tc),_w(1), _tr(0)
{
	canon();
}


int dDd::getTl() const
{
	return _tl;
}
unsigned int dDd::getV() const
{
	return _v;
}

int dDd::getTc() const
{
	return _tc;
}

unsigned int dDd::getW() const
{
	return _w;
}

int dDd::getTr() const
{
	return _tr;
}

bool dDd::operator>=(const dDd & m) const
{
	return m <= (*this);
}

bool dDd::operator<=(const dDd & m) const
{
	return getRw() <= m.getRw();
}

bool dDd::operator==(const dDd & m) const
{
	if (_v == m._v && _w==m._w && _tl == m._tl && _tr == m._tr && _tc == m._tc)
	{
		return true;
	}
	else return false;
}

void dDd::canon()
{

    switch(_canon)
    {
    case 0:
        canonL();
        break;
    case 1:
        canonC();
        break;
    default:
        canonR();
        break;
    }
}


void dDd::canonL()
{
	_tr += _w * _tc;
	_tc = 0;
	if (_tr >= 0)
	{
		_tl += (_tr / _w)*_v;
		_tr = _tr % _w;
		if (_tr != 0)
		{
			_tr -= _w;
			_tl += _v;
		}
	}
	else
	{
		int abstr = -_tr;
		int n = (abstr / _w)*_v;
		abstr = abstr % _w;
		_tr = -abstr;
		_tl -= n;
	}
}

void dDd::canonR()
{
	_tl += _v * _tc;
	_tc = 0;
	if (_tl >= 0)
	{
		_tr += (_tl / _v)*_w;
		_tl = _tl % _v;
		if (_tl != 0)
		{
			_tl -= _v;
			_tr += _w;
		}
	}
	else
	{
		int abstl = -_tl;
		int n = (abstl / _v)*_w;
		abstl = abstl % _v;
		_tl = -abstl;
		_tr -= n;
	}
}

void dDd::canonC()
{
	if (_tr >= 0)
	{
		_tc += (_tr / _w);
		_tr = _tr % _w;
		if (_tr != 0)
		{
			_tr -= _w;
			_tc += 1;
		}
	}
	else
	{
		int abstr = -_tr;
		int n = (abstr / _w);
		abstr = abstr % _w;
		_tr = -abstr;
		_tc -= n;
	}

	if (_tl >= 0)
	{
		_tc += (_tl / _v);
		_tl = _tl % _v;
		if (_tl != 0)
		{
			_tl -= _v;
			_tc += 1;
		}
	}
	else
	{
		int abstl = -_tl;
		int n = (abstl / _v);
		abstl = abstl % _v;
		_tl = -abstl;
		_tc -= n;
	}
}

int dDd::Rw(int ti) const
{	
    int res = (((int)ceil((ti+_tr)/(double)_w)+_tc) *_v)+_tl;
    return res;
}

Fmaxp dDd::getRw() const
{
	vector<int> seq(this->_w);
	for (unsigned i = 0; i<seq.size(); i++)
	{
		seq[i] = Rw(i);
	}
	return Fmaxp(_w, _v, seq);
}

std::pair<unsigned,unsigned> dDd::getPeriodicity() const
{
    return pair<unsigned,unsigned>(_w,_v);
}


std::string dDd::toString(unsigned nVer) const
{
    std::stringstream ss;
    if(nVer==0)
    {
        if(getPeriodicity()==pair<unsigned,unsigned>(1,1)){    ss<<"d"<<_tl+_tc+_tr;  }
        else
        {
          bool pre = false;
          if (_tl)
          {
            ss << "d" << _tl;
            pre = true;
          }

		  if (_v != 1)
		  {
			  if (pre) ss << ".";
			  ss << "v" << _v;
			  pre = true;
		  }

          if(_tc)
          {
			 if (pre) ss << ".";
             if(_tc) ss<<"d"<<_tc << "";             
             pre = true;
           }
		
		if (_w != 1)
		{
			if (pre) ss << ".";
			ss << "w" << _w;
			pre = true;
		}
		
		if (_tr)
         {
             if(pre) ss << ".";
             ss << "d" << _tr;
         }
        }
    }
     else
     {
             if(getPeriodicity()==pair<unsigned,unsigned>(1,1)){   ss<<"d"<<_tl+_tc+_tr; }
             else
             {
                if(_tl) ss<<"d"<<_tl;
                ss << "(" << _v << ":";
                if(_tc) ss << _tc << ":";
                ss << _w<<")";
                if(_tr) ss <<"d"<<_tr;
             }
     }
    return ss.str();
}



T_op dDd::extendBy(unsigned mul) const
{
	T_op op(*this);
	return op.extendBy(mul);
}

/*  TODO : add this extension (see the version in gNg)
Eop gNg::ExtendBy(unsigned mul) const
{
    gNg cp(*this);
    cp.CanonL();

    if(mul>=2)
    {
        Eop op(gNg(0+cp._nl,mul*cp._m,mul*cp._b,(mul-1)*cp._b+cp._nr));
        for(int i=1;i<mul;i++)
        {
            op.Add(gNg(cp._nl+i*cp._m,mul*cp._m,mul*cp._b,(mul-1-i)*cp._b+cp._nr));
        }
        return op;
    }
    else return Eop(*this);
}*/

 std::ostream & operator<<(std::ostream & f, const dDd & m)
 {
     f<<m.toString();
     return f;
 }

}
