#include "seriesET.h"
#include "../common/etvoTools.h"
#include<vector>
#include "../etop/E_op.h"
#include "../etop/T_op.h"
#include "../common/etvoException.h"
#include "../common/global.h"

#include<algorithm>
#include<sstream>

//#define TRACE

using namespace std;
namespace etvo
{	
	// gamma operator as seriesET
	seriesET etg(int n, bool QEdForm) 
	{ 
		if (QEdForm)
		{
			matrix<dDd> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = dDd(0);
			matrix<seriesEd> Q(1, 1);
			Q(0, 0) = Ed::g(n);
			return seriesET(L,Q,R);
		}
		else
		{
			matrix<gNg> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = gNg(0);
			matrix<seriesTg> Q(1, 1);
			Q(0, 0) = Tg::g(n);
			return seriesET(L, Q, R);
		}
	}

	//delta operator as seriesET
	seriesET etd(int t, bool QEdForm)
	{
		if (QEdForm)
		{
			matrix<dDd> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = dDd(0);
			matrix<seriesEd> Q(1, 1);
			Q(0, 0) = Ed::d(t);
			return seriesET(L, Q, R);
		}
		else
		{
			matrix<gNg> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = gNg(0);
			matrix<seriesTg> Q(1, 1);
			Q(0, 0) = Tg::d(t);
			return seriesET(L, Q, R);
		}
	}

	// mu operator (event mul) as serieET
	seriesET etm(unsigned m, bool QEdForm)
	{
		if (QEdForm)
		{
			matrix<dDd> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = dDd(0);
			matrix<seriesEd> Q(1, 1);
			Q(0, 0) = Ed::m(m);
			return seriesET(L, Q, R);
		}
		else
		{
			if (m < 1) m = 1;
			matrix<gNg> L(1, m), R(1, 1);
			R(0, 0) = gNg(0);
			for (int j = 0; j < m; j++) { L(0, j) = gNg(j, m, 1, 0); }
			matrix<seriesTg> Q(m, 1);
			Q(0, 0) = Tg::E();
			return seriesET(L, Q, R);
		}
	}

	seriesET etm(const std::vector<unsigned> & seq)
	{
		return seriesET(em(seq));
	}

	seriesET etb(const std::vector<unsigned> & seq)
	{
		return seriesET(eb(seq));
	}

	seriesET etd(const std::vector<int> & seqDelays)
	{
		return seriesET(td(seqDelays));
	}


	//beta operator as seriesET (event div)
	seriesET etb(unsigned b, bool QEdForm)
	{
		if (QEdForm)
		{
			matrix<dDd> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = dDd(0);
			matrix<seriesEd> Q(1, 1);
			Q(0, 0) = Ed::b(b);
			return seriesET(L, Q, R);
		}
		else
		{
			if (b < 1) b = 1;
			matrix<gNg> L(1, 1), R(b, 1);
			L(0, 0) = gNg(0);
			for (int j = 0; j < b; j++) { R(j,0) = gNg(0, 1, b, b-1-j); }
			matrix<seriesTg> Q(1,b);
			Q(0, b-1) = Tg::E();
			return seriesET(L, Q, R);
		}
	}

	// Vee operator (Time multiplier) as seriesET
	seriesET etv(unsigned v, bool QEdForm) {
		if (!QEdForm) //Qtg form
		{
			matrix<gNg> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = gNg(0);
			matrix<seriesTg> Q(1, 1);
			Q(0, 0) = Tg::V(v);
			return seriesET(L, Q, R);
		}
		else
		{
			if (v < 1) v = 1;
			matrix<dDd> L(1, v), R(1, 1);
			R(0, 0) = dDd(0);
			for (int j = 0; j < v; j++) { L(0, j) = dDd(-j, v, 1, 0); }
			matrix<seriesEd> Q(v, 1);
			Q(0, 0) = Ed::E();
			return seriesET(L, Q, R);
		}
	}
	// Wedge operator (Time divider) as seriesET
	seriesET etw(unsigned w, bool QEdForm) {
		if (!QEdForm) //QtgForm
		{
			matrix<gNg> L(1, 1), R(1, 1);
			L(0, 0) = R(0, 0) = gNg(0);
			matrix<seriesTg> Q(1, 1);
			Q(0, 0) = Tg::W(w);
			return seriesET(L, Q, R);
		}
		else
		{
			if (w < 1) w = 1;
			matrix<dDd> L(1, 1), R(w, 1);
			L(0, 0) = dDd(0);
			for (int j = 0; j < w; j++) { R(j, 0) = dDd(0, 1, w, 1 - (int)(w)+j); }
			matrix<seriesEd> Q(1, w);
			Q(0, w - 1) = Ed::E();
			return seriesET(L, Q, R);
		}
	}
		
	void seriesET::toLeft()
	{
		if (_QEdForm)
		{
			for (unsigned i = 0; i < _Qed.GetRow(); i++)
				for (unsigned j = 0; j < _Qed.GetColumn(); j++)
					_Qed(i, j).toLeft();
		}
		else
		{
			for (unsigned i = 0; i < _Qtg.GetRow(); i++)
				for (unsigned j = 0; j < _Qtg.GetColumn(); j++)
					_Qtg(i, j).toLeft();
		}
	}

	void seriesET::toRight()
	{
		if (_QEdForm)
		{
			for (unsigned i = 0; i < _Qed.GetRow(); i++)
				for (unsigned j = 0; j < _Qed.GetColumn(); j++)
					_Qed(i, j).toRight();
		}
		else
		{
			for (unsigned i = 0; i < _Qtg.GetRow(); i++)
				for (unsigned j = 0; j < _Qtg.GetColumn(); j++)
					_Qtg(i, j).toRight();
		}
	}


	void seriesET::canon()
	{		
			if (_QEdForm)
			{
				auto Nl = getMatNed(_Qed.GetRow());
				auto Nr = getMatNed(_Qed.GetColumn());
				_Qed = Nl * _Qed*Nr;
			}
			else
			{
				auto Nl = getMatNtg(_Qtg.GetRow());
				auto Nr = getMatNtg(_Qtg.GetColumn());
				_Qtg = Nl * _Qtg*Nr;
			}		
	}	

	// returns the specific N matrix for a Core Decomposition in Vee/Wedge	
	etvo::matrix<seriesEd>  seriesET::getMatNed(unsigned size)
	{
		// in the Qed form, L and R are in T[[g]] 
		// N is the same as in the core decomposition of seriesTg
		// but N must be expressed as seriesEd (homogeneous with Qed)		
		etvo::matrix<seriesEd>  Nm(size, size);
		for (unsigned i = 0; i < size; i++)
		{
			for (unsigned j = 0; j < size; j++)
			{
				if (j <= i) Nm(i, j) = seriesEd::E();
				else Nm(i, j) = seriesEd(Ed::d(-1));
			}
		}
		return Nm;
	}

	// returns the specific N matrix for a Core Decomposition in Mu/Beta	
	etvo::matrix<seriesTg> seriesET::getMatNtg(unsigned size)
	{
		// in the Qtg form, L and R are in E[[d]] 
		// N is the same as in the core decomposition of seriesEd
		// but N must be expressed as seriesTg (homogeneous with Qtg)	
		etvo::matrix<seriesTg>  Em(size, size);
		for (unsigned i = 0; i < size; i++)
		{
			for (unsigned j = 0; j < size; j++)
			{
				if (j <= i) Em(i, j) = seriesTg::E();
				else Em(i, j) = seriesTg(Tg::g(1));
			}
		}
		return Em;
	}
	
	seriesET::seriesET():ISterm(true),_QEdForm(true),
			_Led(0,0),_Qed(0,0),_Red(0,0),
			_Ltg(0, 0), _Qtg(0, 0), _Rtg(0, 0)
	{}

	seriesET::seriesET(bool TopNotE): _QEdForm(true), 
		_Led(0, 0), _Qed(0, 0), _Red(0, 0),
		_Ltg(0, 0), _Qtg(0, 0), _Rtg(0, 0)
	{
		if (TopNotE){		setTop();}
		else{		*this = E();	}
	}

	seriesET::seriesET(const matrix<dDd> & Led, 
					const matrix<seriesEd> Qed, 
					const matrix<dDd> & Red): _QEdForm(true), 
				 _Led(Led), _Qed(Qed),_Red(Red),
		_Ltg(0, 0), _Qtg(0, 0), _Rtg(0, 0)
	{}

	seriesET::seriesET(const matrix<gNg> & Ltg, 
						const matrix<seriesTg> Qtg, 
						const matrix<gNg> & Rtg) : _QEdForm(false),
		_Led(0,0), _Qed(0,0), _Red(0,0),
		_Ltg(Ltg), _Qtg(Qtg), _Rtg(Rtg)
	{}
	
	/// Initialization of a seriesET from seriesEd 
	seriesET::seriesET(const seriesEd & s) : _QEdForm(true),
		_Ltg(0, 0), _Qtg(0, 0), _Rtg(0, 0),
		_Led(1, 1), _Red(1, 1),_Qed(1,1)
	{
		_Led(0,0) = dDd(0);		
		_Red(0,0) = dDd(0); 
		_Qed(0, 0) = s;
	}

	/// Initialization of a seriesET from seriesTg 
	seriesET::seriesET(const seriesTg & s) : _QEdForm(false),
		_Led(0,0), _Red(0,0), _Qed(0,0),
		_Ltg(1, 1), _Qtg(1, 1), _Rtg(1, 1)
	{
		_Ltg(0, 0) = gNg(0);
		_Rtg(0, 0) = gNg(0);
		_Qtg(0, 0) = s;
	}

	seriesET seriesET::Epsilon() { return seriesET(); }

	seriesET seriesET::Top() { return seriesET(true); }

	seriesET seriesET::E() { return etg(0); }
	   
	/// getter returning Led
	matrix<dDd> seriesET::getLed() const{	return _Led;	}
	/// getter returning Red
	matrix<dDd> seriesET::getRed() const { return _Red; }
	/// getter returning Qed 
	matrix<seriesEd> seriesET::getQed() const { return _Qed; }

	/// getter returning Ltg
	matrix<gNg> seriesET::getLtg() const { return _Ltg; }
	/// getter returning Rtg
	matrix<gNg> seriesET::getRtg() const { return _Rtg; }
	/// getter returning Qtg 
	matrix<seriesTg> seriesET::getQtg() const { return _Qtg; }

	/// returns the Least Common multiple of gains in the terms of the current series
	void seriesET::getLcmGainEd(unsigned int & mu, unsigned int& beta) const
	{
		mu = 1; 	beta = 1;
		unsigned int M, B;
		if (_QEdForm){
			if (!isEpsilon() && !isTop()){
				for (unsigned l = 0; l < _Qed.GetRow(); l++){
					for (unsigned c = 0; c < _Qed.GetColumn(); c++){
						_Qed(l, c).getLcmGain(M, B);
						mu = Tools::lcm(mu, M);
						beta = Tools::lcm(beta, B);
					}
				}
			}
		}
		else
		{
			if (!isEpsilon() && !isTop())
			{
				mu = _Ltg(0, 0).getM();
				beta = _Rtg(0, 0).getB();
			}
		}
	}

	/// returns the Least Common multiple of gains in the terms of the current series
	void seriesET::getLcmGainTg(unsigned int & vee, unsigned int& wedge) const
	{
		vee = 1; 	wedge = 1;
		unsigned int V,W;
		if (!_QEdForm) { //_Qtg form
			if (!isEpsilon() && !isTop()) {
				for (unsigned l = 0; l < _Qtg.GetRow(); l++) {
					for (unsigned c = 0; c < _Qtg.GetColumn(); c++) {
						_Qtg(l, c).getLcmGain(V, W);
						vee = Tools::lcm(vee, V);
						wedge = Tools::lcm(wedge, W);
					}
				}
			}
		}
		else
		{
			if (!isEpsilon() && !isTop())
			{
				vee = _Led(0, 0).getV();
				wedge = _Red(0, 0).getW();
			}
		}
	}

	/// returns the Least Common multiple of ultimate slopes (g^n d^t)* of Q terms
	void seriesET::getLcmQSlope(int & g, int & d) const
	{
		g = 1, d = 1;
		int gamma = 1, delta = 1;
		bool oneSeries = false;
		if (!isEpsilon() && !isTop())
		{
			if (_QEdForm){
				for (unsigned l = 0; l < _Qed.GetRow(); l++) {
					for (unsigned c = 0; c < _Qed.GetColumn(); c++)
					{
						if (!_Qed(l, c).isExtreme())
						{
							auto sTmp=_Qed(l, c);
							sTmp.toRight();							
							gamma = sTmp.getR().getg();
							delta = sTmp.getR().getd();							
							if (delta != 0) //not a polynimial
							{
								oneSeries = true;
								g = Tools::lcm(g, gamma);
								d = Tools::lcm(d, delta);
							}
						}
					}
				}
			}
			else
			{
				for (unsigned l = 0; l < _Qtg.GetRow(); l++) {
					for (unsigned c = 0; c < _Qtg.GetColumn(); c++) {
						if (!_Qtg(l, c).isExtreme())
						{
							auto sTmp = _Qtg(l, c);
							sTmp.toRight();
							gamma = sTmp.getR().getg();
							delta = sTmp.getR().getd();
							if (delta != 0) //not a polynimial
							{
								oneSeries = true;
								g = Tools::lcm(g, gamma);
								d = Tools::lcm(d, delta);
							}
						}
					}
				}
			}
			if (!oneSeries) { g = 1; d = 0; }
		}
	}

	bool seriesET::isPolynomial() const
	{
		if (_QEdForm)
		{
			for (unsigned l = 0; l < _Qed.GetRow(); l++) {
				for (unsigned c = 0; c < _Qed.GetColumn(); c++) {
					if (!_Qed(l, c).isPolynomial()) return false;
				}
			}
			return true;
		}
		else
		{
			for (unsigned l = 0; l < _Qtg.GetRow(); l++) {
				for (unsigned c = 0; c < _Qtg.GetColumn(); c++) {
					if (!_Qtg(l, c).isPolynomial()) return false;
				}
			}
			return true;
		}
	}

	

	void seriesET::toQtg_polynomial_case()
	{
		unsigned int M, B;
		getLcmGainEd(M, B);
		_Ltg = matrix<gNg>(1, M);
		for (int x = 0; x < M; x++) _Ltg(0, x) = gNg(x, M, 1, 0);
		_Rtg = matrix<gNg>(B, 1);
		for (int x = 0; x < B; x++) _Rtg(x, 0) = gNg(0, 1, B, B - 1 - x);
		_Qtg = matrix<seriesTg>(M, B);

		//foreach series Qed_ij 
		for (unsigned r = 0; r < _Qed.GetRow(); r++)
		{
			for (unsigned c = 0; c < _Qed.GetColumn(); c++)
			{
				if (!_Qed(r, c).isEpsilon())
				{
					auto gNgf = gNg::getCanonForm();
					gNg::setCanonForm(1);  // central form for gNg terms
					polyEd pol = _Qed(r, c).getQ();
					for (unsigned i = 0; i < pol.size(); i++)
					{						
						Ed mo = pol[i];
						E_op w;
						pair<unsigned, unsigned> per = mo.getE_op().getPeriodicity();
						if (per.first != B)
						{
							unsigned rat = B / per.first;
							w = mo.getE_op().extendBy(rat);
						}
						else w = mo.getE_op();
						vector<gNg> v = w.getTerms();
						int delta = mo.getD();
						for (gNg term : v)  // all gNg terms in central form
						{
							unsigned  li = term.getNl();
							unsigned  co = B - 1 - term.getNr();
							Tg monom(dDd(_Led(0, r).getTl(),
								_Led(0, r).getV(),
								delta,
								_Red(c, 0).getW(),
								_Red(c, 0).getTr()),
								term.getNc());
							seriesTg s(monom);
							_Qtg(li, co) = s + _Qtg(li, co);
						}
					}
					gNg::setCanonForm(gNgf);
				}
			}
		}
		_QEdForm = false;
		_Qed = matrix<seriesEd>(0, 0);
		_Led = matrix<dDd>(0, 0);
		_Red = matrix<dDd>(0, 0);		
	}

//version 2 6/6/21
void seriesET::toQtg_series_case()
{
	unsigned int M, B;
	getLcmGainEd(M, B);
	unsigned int W = _Red(0, 0).getW();
	_Ltg = matrix<gNg>(1, M);
	for (int x = 0; x < M; x++) _Ltg(0, x) = gNg(x, M, 1, 0);
	_Rtg = matrix<gNg>(B, 1);
	for (int x = 0; x < B; x++) _Rtg(x, 0) = gNg(0, 1, B, B - 1 - x);
	_Qtg = matrix<seriesTg>(M, B);
	auto P = matrix<polyTg>(M, B);
	auto Q = matrix<polyTg>(M, B);
	int gammaSlope, deltaSlope;
	this->getLcmQSlope(gammaSlope, deltaSlope);	
	int ratio = Tools::lcm(gammaSlope, int(B)) / gammaSlope;
	if (ratio > 1)
	{
		gammaSlope = ratio * gammaSlope;
		deltaSlope = ratio * deltaSlope;
	}
	polyEd pTemp, qTemp;
	gd rTemp;

	//foreach series Qed_ij 
	for (unsigned r = 0; r < _Qed.GetRow(); r++) {
		for (unsigned c = 0; c < _Qed.GetColumn(); c++) {
			if (!_Qed(r, c).isEpsilon())
			{
				pTemp = _Qed(r, c).getP();
				if (!_Qed(r, c).isPolynomial())
				{
					_Qed(r, c).toRight();
					gd slope_rc = _Qed(r, c).getR();
					int rat = gammaSlope / slope_rc.getg();
					qTemp = _Qed(r, c).getQ();
					rTemp = gd(gammaSlope, deltaSlope);
					if (rat != 1)
					{
						polyEd rightFactor;
						for (int i = 0; i < rat; i++)
						{
							rightFactor = rightFactor
								+ Ed(E_op::Gamma(i*slope_rc.getg()), i*slope_rc.getd());
						}
						qTemp = qTemp * rightFactor;
					}
				}
				else //_Qed(r,c) is a polynomial
				{
					pTemp = _Qed(r, c).getQ();
				}				
				auto gNgf = gNg::getCanonForm();
				gNg::setCanonForm(1);  // central form for gNg terms
				polyEd pol = pTemp;
				for (unsigned i = 0; i < pol.size(); i++)
				{
					Ed mo = pol[i];
					E_op w;
					pair<unsigned, unsigned> per = mo.getE_op().getPeriodicity();
					if (per.first != B)
					{
						unsigned rat = B / per.first;
						w = mo.getE_op().extendBy(rat);
					}
					else w = mo.getE_op();
					vector<gNg> v = w.getTerms();
					int delta = mo.getD();
					for (gNg term : v)  // all gNg terms in central form
					{
						unsigned  li = term.getNl();
						unsigned  co = B - 1 - term.getNr();
						Tg monom(dDd(_Led(0, r).getTl(),
							_Led(0, r).getV(),
							delta,
							_Red(c, 0).getW(),
							_Red(c, 0).getTr()),
							term.getNc());
						polyTg s(monom);
						P(li, co) = P(li, co)+s;
					}
				}
				if (!_Qed(r, c).isPolynomial())
				{
					pol = qTemp;
					for (unsigned i = 0; i < pol.size(); i++)
					{
						Ed mo = pol[i];
						E_op w;
						pair<unsigned, unsigned> per = mo.getE_op().getPeriodicity();
						if (per.first != B)
						{
							unsigned rat = B / per.first;
							w = mo.getE_op().extendBy(rat);
						}
						else w = mo.getE_op();
						vector<gNg> v = w.getTerms();
						int delta = mo.getD();
						for (gNg term : v)  // all gNg terms in central form
						{
							unsigned  li = term.getNl();
							unsigned  co = B - 1 - term.getNr();
							Tg monom(dDd(_Led(0, r).getTl(),
								_Led(0, r).getV(),
								delta,
								_Red(c, 0).getW(),
								_Red(c, 0).getTr()),
								term.getNc());
							Q(li, co) = Q(li, co)+monom;
						}
					}				
				}
				gNg::setCanonForm(gNgf);
			}
		}
	}
	// matrices P and Q contain informations s_ij=P_ij + Q_ij.(g^gammaSlope/B.d^deltaSlope*W)*
	for (unsigned r = 0; r < _Qtg.GetRow(); r++) {
		for (unsigned c = 0; c < _Qtg.GetColumn(); c++) {
			_Qtg(r, c) = seriesTg(P(r, c), Q(r, c), gd(gammaSlope / B, deltaSlope*W));
		}
	}
	_QEdForm = false;
	_Qed = matrix<seriesEd>(0, 0);
	_Led = matrix<dDd>(0, 0);
	_Red = matrix<dDd>(0, 0);	
}


	void seriesET::toQed_polynomial_case()
	{
		unsigned int V, W;
		getLcmGainTg(V, W);
		_Led = matrix<dDd>(1, V);
		for (int x = 0; x < V; x++) _Led(0, x) = dDd(-x, V, 1, 0);
		_Red = matrix<dDd>(W, 1);
		for (int x = 0; x < W; x++) _Red(x, 0) = dDd(0, 1, W, 1 - (int)(W)+x);
		_Qed = matrix<seriesEd>(V, W);

		//foreach series Qtg_ij 
		for (unsigned r = 0; r < _Qtg.GetRow(); r++)
		{
			for (unsigned c = 0; c < _Qtg.GetColumn(); c++)
			{
				if (!_Qtg(r, c).isEpsilon())
				{
					auto dDdf = dDd::getCanonForm();
					dDd::setCanonForm(1);  // central form for dDd terms
					polyTg pol = _Qtg(r, c).getQ();
					for (unsigned i = 0; i < pol.size(); i++)
					{						
						Tg mo = pol[i];
						T_op w;
						pair<unsigned, unsigned> per = mo.getT_op().getPeriodicity();
						if (per.first != W)
						{
							unsigned rat = W / per.first;
							w = mo.getT_op().extendBy(rat);
						}
						else w = mo.getT_op();
						vector<dDd> v = w.getTerms();
						int gamma = mo.getG();
						for (dDd term : v)  // all gNg terms in central form
						{
							unsigned  li = -term.getTl();
							unsigned  co = unsigned(int(W) - 1 + term.getTr());
							Ed monom(gNg(_Ltg(0, r).getNl(),
								_Ltg(0, r).getM(),
								gamma,
								_Rtg(c, 0).getB(),
								_Rtg(c, 0).getNr()),
								term.getTc());
							seriesEd s(monom);
							_Qed(li, co) = s + _Qed(li, co);
						}
					}
					dDd::setCanonForm(dDdf);
				}
			}
		}
		_QEdForm = true;
		_Qtg = matrix<seriesTg>(0, 0);
		_Ltg = matrix<gNg>(0, 0);
		_Rtg = matrix<gNg>(0, 0);		
	}
	
	//version 2 6/6/21
	void seriesET::toQed_series_case()
	{
		unsigned int V, W;
		getLcmGainTg(V, W);
		unsigned int B = _Rtg(0, 0).getB();
		_Led = matrix<dDd>(1, V);
		for (int x = 0; x < V; x++) _Led(0, x) = dDd(-x, V, 1, 0);
		_Red = matrix<dDd>(W, 1);
		for (int x = 0; x < W; x++) _Red(x, 0) = dDd(0, 1, W, 1 - (int)(W)+x);
		_Qed = matrix<seriesEd>(V, W);
		auto P = matrix<polyEd>(V,W);
		auto Q = matrix<polyEd>(V,W);

		int gammaSlope, deltaSlope;
		this->getLcmQSlope(gammaSlope, deltaSlope);		
		int ratio = Tools::lcm(deltaSlope, int(W)) / deltaSlope;
		if (ratio > 1)
		{
			gammaSlope = ratio * gammaSlope;
			deltaSlope = ratio * deltaSlope;
		}
		polyTg pTemp, qTemp;
		gd rTemp;

		//foreach series Qtg_ij 
		for (unsigned r = 0; r < _Qtg.GetRow(); r++) {
			for (unsigned c = 0; c < _Qtg.GetColumn(); c++) {
				if (!_Qtg(r, c).isEpsilon())
				{
					pTemp = _Qtg(r, c).getP();
					if (!_Qtg(r, c).isPolynomial())
					{
						_Qtg(r, c).toRight();
						gd slope_rc = _Qtg(r, c).getR();
						int rat = deltaSlope / slope_rc.getd();
						qTemp = _Qtg(r, c).getQ();
						rTemp = gd(gammaSlope, deltaSlope);
						if (rat != 1)
						{
							polyTg rightFactor;
							for (int i = 0; i < rat; i++)
							{
								rightFactor = rightFactor
									+ Tg(T_op::delta(i*slope_rc.getd()), i*slope_rc.getg());
							}
							qTemp = qTemp * rightFactor;
						}
					}
					else
					{
						pTemp = _Qtg(r, c).getQ();
					}					
					auto dDdf = dDd::getCanonForm();
					dDd::setCanonForm(1);  // central form for dDd terms
					polyTg pol = pTemp;
					for (unsigned i = 0; i < pol.size(); i++)
					{
						Tg mo = pol[i];
						T_op w;
						pair<unsigned, unsigned> per = mo.getT_op().getPeriodicity();
						if (per.first != W)
						{
							unsigned rat = W / per.first;
							w = mo.getT_op().extendBy(rat);
						}
						else w = mo.getT_op();
						vector<dDd> v = w.getTerms();
						int gamma = mo.getG();
						for (dDd term : v)  // all dDd terms in central form
						{
							unsigned  li = -term.getTl();
							unsigned  co = unsigned(int(W) - 1 + term.getTr());
							Ed monom(gNg(_Ltg(0, r).getNl(),
								_Ltg(0, r).getM(),
								gamma,
								_Rtg(c, 0).getB(),
								_Rtg(c, 0).getNr()),
								term.getTc());							
							P(li, co) = P(li, co)+monom;
						}
					}
					if (!_Qtg(r, c).isPolynomial())
					{
						pol = qTemp;
						for (unsigned i = 0; i < pol.size(); i++)
						{
							Tg mo = pol[i];
							T_op w;
							pair<unsigned, unsigned> per = mo.getT_op().getPeriodicity();
							if (per.first != W)
							{
								unsigned rat = W / per.first;
								w = mo.getT_op().extendBy(rat);
							}
							else w = mo.getT_op();
							vector<dDd> v = w.getTerms();
							int gamma = mo.getG();
							for (dDd term : v)  // all dDd terms in central form
							{
								unsigned  li = -term.getTl();
								unsigned  co = unsigned(int(W) - 1 + term.getTr());
								Ed monom(gNg(_Ltg(0, r).getNl(),
									_Ltg(0, r).getM(),
									gamma,
									_Rtg(c, 0).getB(),
									_Rtg(c, 0).getNr()),
									term.getTc());
								//seriesEd s(polyEd(), polyEd(monom), gd(gammaSlope*B, deltaSlope / W));
								Q(li, co) = Q(li, co) + monom;
							}
						}
					}
					dDd::setCanonForm(dDdf);
				}
			}
		}
		// s_ij=P_ij+Q_ij.(g^gammaSLope*Bd^deltaSlope/W)*
		for (unsigned r = 0; r < _Qed.GetRow(); r++) {
			for (unsigned c = 0; c < _Qed.GetColumn(); c++) {
				_Qed(r, c) = seriesEd(P(r, c), Q(r, c), gd(gammaSlope*B, deltaSlope / W));
			}
		}
		_QEdForm = true;
		_Qtg = matrix<seriesTg>(0, 0);
		_Ltg = matrix<gNg>(0, 0);
		_Rtg = matrix<gNg>(0, 0);		
	}
		
	//from Qed form to Qtg form
	void seriesET::toCoreTg(){
		// the main issue : from a centered Qed form to a centered Qtg one
		//TODO
		// toCenter(); 

		// assumed centered
		if (_QEdForm)
		{   // case all terms in Qed are polynomials 
			if (isPolynomial())		toQtg_polynomial_case();			
			else	toQtg_series_case();
		}
	}
	
	//from Qtg form to Qed form
	void seriesET::toCoreEd()
	{
		// the main issue : from a centered Qed form to a centered Qtg one
		//TODO
		// toCenter();  

		// assumed centered
		if (!_QEdForm)
		{   // case all terms in Qed are polynomials 
			if (isPolynomial())	toQed_polynomial_case();			
			else toQed_series_case();
		}	
	}

	void seriesET::changeForm()
	{
		if (isCoreEd()) toCoreTg();
		else toCoreEd();
	}

	// gives beta_n g^ga s mu_n (it's a truncation of exponents in gamma) 
	seriesTg seriesET::truncTg(const seriesTg & s, int n, int ga)
	{
		if (!s.isPolynomial())
		{
			auto copyS = s;
			copyS.toRight();
			polyTg copyQ = copyS.getQ(), copyP = copyS.getP();
			polyTg pRes, qRes;
			gd rRes;
			gd r = copyS.getR();
			int kappa = Tools::lcm(n, (int)r.getg());
			int rat = kappa / r.getg();
			gd rTemp = gd(rat*r.getg(), rat*r.getd());

			//p+q.(g^kd^t)* extended p + q.(e+g^kd^t+...)(g^Kd^T)* so that K is a multiple of n
			if (rat != 1) {
				polyTg rightFactor;
				for (int i = 0; i < rat; i++) {
					rightFactor = rightFactor + Tg(T_op::delta(i*r.getd()), i*r.getg());
				}
				copyQ = copyQ * rightFactor;
			}
			rRes = gd(kappa / n, rTemp.getd());  // the result is pRes+qRes.(rRes)*

			for (unsigned i = 0; i < copyP.size(); i++)
			{
				Tg m(copyP[i].getT_op(), (int)floor((double)(copyP[i].getG() + ga) / n));
				pRes.add(m);
			}

			for (unsigned i = 0; i < copyQ.size(); i++)
			{
				Tg m(copyQ[i].getT_op(), (int)floor((double)(copyQ[i].getG() + ga) / n));
				qRes.add(m);
			}
			return seriesTg(pRes, qRes, rRes);
		}
		else
		{
			auto copyS = s;
			polyTg copyQ = copyS.getQ();
			polyTg qRes;			
			for (unsigned i = 0; i < copyQ.size(); i++)
			{
				Tg m(copyQ[i].getT_op(), (int)floor((double)(copyQ[i].getG() + ga) / n));
				qRes.add(m);
			}
			return seriesTg(qRes);
		}
	}
	
	// gives w_n d^de s v_n (it's a truncation of exponents in delta) 
	seriesEd seriesET::truncEd(const seriesEd & s, int n, int de)
	{
		if (!s.isPolynomial())
		{
			auto copyS = s;
			copyS.toRight();
			polyEd copyQ = copyS.getQ(), copyP = copyS.getP();
			polyEd pRes, qRes;
			gd rRes;
			gd r = copyS.getR();
			int tau = Tools::lcm(n, (int)r.getd());
			int rat = tau / r.getd();
			gd rTemp = gd(rat*r.getg(), rat*r.getd());
			if (rat != 1) {
				polyEd rightFactor;
				for (int i = 0; i < rat; i++) {
					rightFactor = rightFactor + Ed(E_op::Gamma(i*r.getg()), i*r.getd());
				}
				copyQ = copyQ * rightFactor;
			}
			rRes = gd(rTemp.getg(), tau / n);

			for (unsigned i = 0; i < copyP.size(); i++)
			{
				Ed m(copyP[i].getE_op(), (int)ceil((double)(copyP[i].getD() + de) / n));
				pRes.add(m);
			}

			for (unsigned i = 0; i < copyQ.size(); i++)
			{
				Ed m(copyQ[i].getE_op(), (int)ceil((double)(copyQ[i].getD() + de) / n));
				qRes.add(m);
			}
			return seriesEd(pRes, qRes, rRes);
		}
		else
		{
			auto copyS = s;
			polyEd copyQ = copyS.getQ();
			polyEd qRes;			
			for (unsigned i = 0; i < copyQ.size(); i++)
			{
				Ed m(copyQ[i].getE_op(), (int)ceil((double)(copyQ[i].getD() + de) / n));
				qRes.add(m);
			}
			return seriesEd(qRes);
		}
	}
	  
	void seriesET::extendQtg(unsigned mul)
	{
		if (mul < 1) mul = 1;
		auto Qhat = getMatNtg(_Qtg.GetRow())*_Qtg*getMatNtg(_Qtg.GetColumn());
		matrix<seriesTg> Qextended(Qhat.GetRow()*mul, Qhat.GetColumn()*mul);
		//cout << Qhat << endl;
		//cout << Qextended.GetRow() << "x" << Qextended.GetColumn() << endl;

		for (int num = 0; num <= 2 * (mul - 1); num++)
		{	//Qtmp = beta_n g^num s mu_n
			matrix<seriesTg> Qtmp(Qhat.GetRow(), Qhat.GetColumn());
			for (unsigned i = 0; i < Qtmp.GetRow(); i++){
				for (unsigned j = 0; j < Qtmp.GetColumn(); j++) Qtmp(i, j) = truncTg(Qhat(i, j), (int)mul, num);				
			}

			for (int l = 0; l < mul; l++)
			{
				for (int c = 0; c < mul; c++)
				{
					if ((l + c) == num)
					{
						//indices de dbut dans le rsultat 
						int a = (mul - 1 - l)*Qhat.GetRow();
						int b = c * Qhat.GetColumn();
						//cout << "ligne debut=" << a  << endl;
						//cout << "col dbut=" << b << endl;
						//recopier tous les termes de Qtmp dans un bloc du rsultat 
						for (int i = 0; i < Qtmp.GetRow(); i++)
						{
							for (int j = 0; j < Qtmp.GetColumn(); j++)
							{
								Qextended(a + i, b + j) = Qtmp(i, j);
							}
						}
					}
				}
			}
		}
		matrix<gNg> Lext(1, Qextended.GetRow());
		for(int i=0;i< Qextended.GetRow();i++) 
			Lext(0,i)= gNg(i, Qextended.GetRow(), 1, 0);
		matrix<gNg> Rext(Qextended.GetColumn(),1);
		for (int i = 0; i < Qextended.GetColumn(); i++)
			Rext(i,0) = gNg(0,1,Qextended.GetColumn(), (int)(Qextended.GetColumn())-1-i);
		_Ltg = Lext;
		_Rtg = Rext;
		_Qtg = Qextended;
	}
		
	void seriesET::extendQed(unsigned mul)
	{   // L and R are in T[[g]] 		
		if (mul < 1) mul = 1;
		auto Qhat = getMatNed(_Qed.GetRow())*_Qed*getMatNed(_Qed.GetColumn());
		matrix<seriesEd> Qextended(Qhat.GetRow()*mul, Qhat.GetColumn()*mul);
		//cout << Qhat << endl;
		//cout << Qextended.GetRow() << "x" << Qextended.GetColumn() << endl;

		for (int num = 0; num <= 2 * (mul - 1); num++)
		{	//Qtmp = w_n d^(-num) s v_n
			matrix<seriesEd> Qtmp(Qhat.GetRow(), Qhat.GetColumn());
			for (unsigned i = 0; i < Qtmp.GetRow(); i++) {
				for (unsigned j = 0; j < Qtmp.GetColumn(); j++) 
					Qtmp(i, j) = truncEd(Qhat(i, j), (int)mul, -num);
			}

			for (int l = 0; l < mul; l++)
			{
				for (int c = 0; c < mul; c++)
				{
					if ((l + c) == num)
					{
						//indices de dbut dans le rsultat 
						int a = (mul - 1 - l)*Qhat.GetRow();
						int b = c * Qhat.GetColumn();
						//cout << "ligne debut=" << a  << endl;
						//cout << "col dbut=" << b << endl;
						//recopier tous les termes de Qtmp dans un bloc du rsultat 
						for (int i = 0; i < Qtmp.GetRow(); i++)
						{
							for (int j = 0; j < Qtmp.GetColumn(); j++)
							{
								Qextended(a + i, b + j) = Qtmp(i, j);
							}
						}
					}
				}
			}
		}
		matrix<dDd> Lext(1, Qextended.GetRow());
		for (int i = 0; i < Qextended.GetRow(); i++)
			Lext(0, i) = dDd(-i, Qextended.GetRow(), 1, 0);
		matrix<dDd> Rext(Qextended.GetColumn(), 1);
		for (int i = 0; i < Qextended.GetColumn(); i++)
			Rext(i, 0) = dDd(0, 1, Qextended.GetColumn(), 1-(int)(Qextended.GetColumn()) + i);													
		_Led = Lext;
		_Red = Rext;
		_Qed = Qextended;
	}


	void seriesET::extendBy(unsigned mul)
	{
		if (_QEdForm) extendQed(mul);
		else extendQtg(mul);
	}
	   
	std::ostream & operator<<(std::ostream & f, const seriesET & s)
	{
		f << s.toString();
		return f;
	}  

	seriesET seriesET::oplus(const seriesET & s) const
	{
		if (isTop() || s.isTop()) return Top();
		if (isEpsilon()) return s;
		if (s.isEpsilon()) return *this;
		unsigned int m1, b1, m2, b2,v1,w1,v2,w2;
		getLcmGainEd(m1, b1);
		s.getLcmGainEd(m2, b2);
		getLcmGainTg(v1, w1);
		s.getLcmGainTg(v2, w2);
		if ((m1*b2) != (m2*b1) || (v1*w2)!=(v2*w1) ){
			throw etvoException(310, "Only the sum of seriesET with equal gain is allowed!");
		}
		auto cThis(*this);
		auto cS(s);
		if (isCoreEd() && s.isCoreEd()){			
			long V = Tools::lcm(v1, v2);
			if((V/v1)>1) cThis.extendQed(V / v1);
			if ((V / v2) > 1) cS.extendQed(V / v2);
			return seriesET(cThis._Led, cThis._Qed + cS._Qed, cThis._Red);
		}

		if (isCoreTg() && s.isCoreTg()){			
			long M = Tools::lcm(m1, m2);
			if ((M / m1) > 1) cThis.extendQtg(M / m1);
			if ((M / m2) > 1) cS.extendQtg(M / m2);
			return seriesET(cThis._Ltg, cThis._Qtg + cS._Qtg, cThis._Rtg);
		}
		// Two different forms		
		bool bothQed = false;
		if (isCoreEd())
		{  // this=Qed s = Qtg
			if ((_Qed.GetRow()*_Qed.GetColumn()) < (s._Qtg.GetRow()*s._Qtg.GetColumn()))
			{
				cS.toCoreEd(); bothQed = true;
			}
			else cThis.toCoreTg();  //both Qtg
		}
		else
		{
			if ((_Qtg.GetRow()*_Qtg.GetColumn()) < (s._Qed.GetRow()*s._Qed.GetColumn()))
				cS.toCoreTg();
			else {	cThis.toCoreEd(); bothQed = true;	}
		}

		if (bothQed)
		{
			long V = Tools::lcm(v1, v2);
			if ((V / v1) > 1) cThis.extendQed(V / v1);
			if ((V / v2) > 1) cS.extendQed(V / v2);
			return seriesET(cThis._Led, cThis._Qed + cS._Qed, cThis._Red);
		}
		else
		{
			long M = Tools::lcm(m1, m2);
			if ((M / m1) > 1) cThis.extendQtg(M / m1);
			if ((M / m2) > 1) cS.extendQtg(M / m2);
			return seriesET(cThis._Ltg, cThis._Qtg + cS._Qtg, cThis._Rtg);
		}
	}

	seriesET seriesET::otimes(const seriesET & s) const
	{
		if (this->isEpsilon()) return seriesET::Epsilon();  // retourne eps
		if (s.isEpsilon()) return seriesET::Epsilon();  // retourne eps
		if (this->isTop()) return seriesET::Top();  // retourne T
		if (s.isTop()) return seriesET::Top();  // retourne T
		unsigned int m1, b1, m2, b2, v1, w1, v2, w2;
		getLcmGainEd(m1, b1);
		s.getLcmGainEd(m2, b2);
		getLcmGainTg(v1, w1);
		s.getLcmGainTg(v2, w2);				
		auto cThis(*this);
		auto cS(s);

		if (cThis.isCoreEd() && cS.isCoreEd())
		{
			long mul = Tools::lcm(w1, v2);
			if ((mul / w1) > 1) cThis.extendQed(mul / w1);
			if ((mul / v2) > 1) cS.extendQed(mul / v2);
			return seriesET(cThis._Led, 
				cThis._Qed*getMatNed(cThis._Qed.GetColumn())*cS._Qed, cS._Red);
		}

		if (cThis.isCoreTg() && cS.isCoreTg())
		{
			long mul = Tools::lcm(b1, m2);
			if ((mul / b1) > 1) cThis.extendQtg(mul / b1);
			if ((mul / m2) > 1) cS.extendQtg(mul / m2);
			return seriesET(cThis._Ltg,
				cThis._Qtg*getMatNtg(cThis._Qtg.GetColumn())*cS._Qtg, cS._Rtg);
		}

		// both are with different forms
		if (m1*b2 < v1*w2)
		{
			// we try to minimze the size of the Core matrix of the result
			//choose Qtg form for both
			if (cThis.isCoreEd()) cThis.toCoreTg();
			else cS.toCoreTg();
			long mul = Tools::lcm(b1, m2);
			if ((mul / b1) > 1) cThis.extendQtg(mul / b1);
			if ((mul / m2) > 1) cS.extendQtg(mul / m2);
			return seriesET(cThis._Ltg,
				cThis._Qtg*getMatNtg(cThis._Qtg.GetColumn())*cS._Qtg, cS._Rtg);
		}
		else
		{
			//choose Qed form for both
			if (cThis.isCoreEd()) cS.toCoreEd();
			else cThis.toCoreEd();
			long mul = Tools::lcm(w1, v2);
			if ((mul / w1) > 1) cThis.extendQed(mul / w1);
			if ((mul / v2) > 1) cS.extendQed(mul / v2);
			return seriesET(cThis._Led,
				cThis._Qed*getMatNed(cThis._Qed.GetColumn())*cS._Qed, cS._Red);
		}
	}

	seriesET seriesET::star() const
	{
		if (isEpsilon()) return seriesET::E();
		unsigned int m1, b1,v1,w1;
		this->getLcmGainEd(m1,b1);
		this->getLcmGainTg(v1, w1);

		if (m1 != b1 || v1!=w1)
		{
			throw etvoException(400, "Impossible to compute the Kleene star of series if gain !=1");
		}
		seriesET cThis(*this);
		if (m1*b1 < v1*w1)	cThis.toCoreTg();
		else	cThis.toCoreEd();

		if (cThis.isCoreEd()){			
			matrix<seriesEd> N = seriesET::getMatNed(cThis._Qed.GetRow());
			matrix<seriesEd> Qk = cThis._Qed * N;
			cThis._Qed = Qk.star();
			return cThis;
		}
		else{
			matrix<seriesTg> N = seriesET::getMatNtg(cThis._Qtg.GetRow());
			matrix<seriesTg> Qk = cThis._Qtg * N;
			cThis._Qtg = Qk.star();
			return cThis;
		}		
	}

	seriesET seriesET::operator+(const seriesET & s) const
	{
		return oplus(s);
	}
	
	seriesET seriesET::operator*(const seriesET & s) const
	{
		return otimes(s);
	}

	seriesET seriesET::inf(const seriesET & s) const
	{
		if (isEpsilon() || s.isEpsilon()) return seriesET::Epsilon();
		if (isTop()) return s;
		if (s.isTop()) return *this;
		unsigned int m1, b1, m2, b2, v1, w1, v2, w2;
		getLcmGainEd(m1, b1);
		s.getLcmGainEd(m2, b2);
		getLcmGainTg(v1, w1);
		s.getLcmGainTg(v2, w2);
		if ((m1*b2) != (m2*b1) || (v1*w2) != (v2*w1)) {
			throw etvoException(310, "Only the sum of seriesET with equal gain is allowed!");
		}
		auto cThis(*this);
		auto cS(s);
		if (isCoreEd() && s.isCoreEd()) {
			long V = Tools::lcm(v1, v2);
			if ((V / v1) > 1) cThis.extendQed(V / v1);
			if ((V / v2) > 1) cS.extendQed(V / v2);
			return seriesET(cThis._Led, cThis._Qed.inf(cS._Qed), cThis._Red);
		}

		if (isCoreTg() && s.isCoreTg()) {
			long M = Tools::lcm(m1, m2);
			if ((M / m1) > 1) cThis.extendQtg(M / m1);
			if ((M / m2) > 1) cS.extendQtg(M / m2);
			return seriesET(cThis._Ltg, cThis._Qtg.inf(cS._Qtg), cThis._Rtg);
		}
		// Two different forms		
		bool bothQed = false;
		if (isCoreEd())
		{  // this=Qed s = Qtg
			if ((_Qed.GetRow()*_Qed.GetColumn()) < (s._Qtg.GetRow()*s._Qtg.GetColumn()))
			{
				cS.toCoreEd(); bothQed = true;
			}
			else cThis.toCoreTg();  //both Qtg
		}
		else
		{
			if ((_Qtg.GetRow()*_Qtg.GetColumn()) < (s._Qed.GetRow()*s._Qed.GetColumn()))
				cS.toCoreTg();
			else { cThis.toCoreEd(); bothQed = true; }
		}

		if (bothQed)
		{
			long V = Tools::lcm(v1, v2);
			if ((V / v1) > 1) cThis.extendQed(V / v1);
			if ((V / v2) > 1) cS.extendQed(V / v2);
			return seriesET(cThis._Led, cThis._Qed.inf(cS._Qed), cThis._Red);
		}
		else
		{
			long M = Tools::lcm(m1, m2);
			if ((M / m1) > 1) cThis.extendQtg(M / m1);
			if ((M / m2) > 1) cS.extendQtg(M / m2);
			return seriesET(cThis._Ltg, cThis._Qtg.inf(cS._Qtg), cThis._Rtg);
		}
	}

	seriesET seriesET::lfrac(const seriesET & s) const
	{
		if (isTop() || s.isEpsilon()) return seriesET::Top();
		if (isEpsilon() || s.isTop()) return seriesET::Epsilon();
		
		auto cThis(*this);
		auto cS(s);
		if ((cThis.isCoreTg() && cS.isCoreTg()))
		{
			unsigned int m1, b1, m2, b2;
			cThis.getLcmGainEd(m1, b1);
			cS.getLcmGainEd(m2, b2);
			unsigned int mul = Tools::lcm(m1, m2);
			cThis.extendBy(mul / m1);	cThis.canon();
			cS.extendBy(mul / m2);		cS.canon();
			auto Qres = cThis._Qtg.lfrac(cS._Qtg);
			matrix<gNg> Lres(1, Qres.GetRow());
			for (int i = 0; i < Qres.GetRow(); i++)
				Lres(0, i) = gNg(i, Qres.GetRow(), 1, 0);
			matrix<gNg> Rres(Qres.GetColumn(), 1);
			for (int i = 0; i < Qres.GetColumn(); i++)
				Rres(i, 0) = gNg(0, 1, Qres.GetColumn(), (int)(Qres.GetColumn()) - 1 - i);
				return seriesET(Lres, Qres, Rres);
		}
		// all other cases (change to Qed form)
		unsigned int v1, w1, v2, w2;
		cThis.getLcmGainTg(v1, w1);
		cS.getLcmGainTg(v2, w2);
		unsigned int mul = Tools::lcm(v1, v2);
		cThis.toCoreEd();
		cS.toCoreEd();
		cThis.extendBy(mul / v1);
		cThis.canon();
		cS.extendBy(mul / v2);
		cS.canon();
		auto Qres = cThis._Qed.lfrac(cS._Qed);
		matrix<dDd> Lres(1, Qres.GetRow());
		for (int i = 0; i < Qres.GetRow(); i++)
			Lres(0, i) = dDd(-i, Qres.GetRow(), 1, 0);
		matrix<dDd> Rres(Qres.GetColumn(), 1);
		for (int i = 0; i < Qres.GetColumn(); i++)
			Rres(i, 0) = dDd(0, 1, Qres.GetColumn(), 1 - (int)(Qres.GetColumn()) + i);
		return seriesET(Lres, Qres, Rres);		
	}

	seriesET seriesET::rfrac(const seriesET & s) const
	{
		if (isTop() || s.isEpsilon()) return seriesET::Top();
		if (isEpsilon() || s.isTop()) return seriesET::Epsilon();
		unsigned int v1, w1, v2, w2;
		auto cThis(*this);
		auto cS(s);
		if ((cThis.isCoreTg() && cS.isCoreTg()))
		{
			unsigned int m1, b1, m2, b2;
			cThis.getLcmGainEd(m1, b1);
			cS.getLcmGainEd(m2, b2);
			unsigned int mul = Tools::lcm(m1, m2);
			cThis.extendBy(mul / b1);	cThis.canon();
			cS.extendBy(mul / b2);		cS.canon();
			auto Qres = cThis._Qtg.rfrac(cS._Qtg);
			matrix<gNg> Lres(1, Qres.GetRow());
			for (int i = 0; i < Qres.GetRow(); i++)
				Lres(0, i) = gNg(i, Qres.GetRow(), 1, 0);
			matrix<gNg> Rres(Qres.GetColumn(), 1);
			for (int i = 0; i < Qres.GetColumn(); i++)
				Rres(i, 0) = gNg(0, 1, Qres.GetColumn(), (int)(Qres.GetColumn()) - 1 - i);
			return seriesET(Lres, Qres, Rres);
		}
		cThis.getLcmGainTg(v1, w1);
		cS.getLcmGainTg(v2, w2);
		unsigned int mul = Tools::lcm(w1, w2);
		cThis.toCoreEd();
		cS.toCoreEd();
		cThis.extendBy(mul / w1);
		cThis.canon();
		cS.extendBy(mul / w2);
		cS.canon();
		auto Qres = cThis._Qed.rfrac(cS._Qed);
		matrix<dDd> Lres(1, Qres.GetRow());
		for (int i = 0; i < Qres.GetRow(); i++)
			Lres(0, i) = dDd(-i, Qres.GetRow(), 1, 0);
		matrix<dDd> Rres(Qres.GetColumn(), 1);
		for (int i = 0; i < Qres.GetColumn(); i++)
			Rres(i, 0) = dDd(0, 1, Qres.GetColumn(), 1 - (int)(Qres.GetColumn()) + i);
		return seriesET(Lres, Qres, Rres);
	}
	
	seriesTg seriesET::scale(const seriesTg & s, unsigned mul)
	{
		polyTg pres, qres;
		polyTg cP = s.getP(), cQ = s.getQ();
		gd rres;
		for (unsigned i = 0; i<cP.size(); i++)
			pres.add(Tg(cP[i].getT_op(), cP[i].getG()*mul));
		for (unsigned i = 0; i<cQ.size(); i++)
			qres.add(Tg(cQ[i].getT_op(), cQ[i].getG()*mul));
		rres = gd(s.getR().getg()*mul, s.getR().getd());
		if (s.isLeftForm()) return seriesTg(pres, qres, rres, false);
		else return seriesTg(pres, qres, rres, true);
	}

	seriesEd seriesET::scale(const seriesEd & s, unsigned mul)
	{
		polyEd pres, qres;
		polyEd cP = s.getP(), cQ = s.getQ();
		gd rres;
		for (unsigned i = 0; i < cP.size(); i++)
			pres.add(Ed(cP[i].getE_op(), cP[i].getD()*mul));
		for (unsigned i = 0; i < cQ.size(); i++)
			qres.add(Ed(cQ[i].getE_op(), cQ[i].getD()*mul));
		rres = gd(s.getR().getg(), s.getR().getd()*mul);
		if (s.isLeftForm()) return seriesEd(pres, qres, rres,false);
		else return seriesEd(pres, qres, rres, true);
	}
	void seriesET::toLVectorTg(matrix<seriesTg> & LV, matrix<gNg> & R) const
	{
		if (!isEpsilon() && !isTop())
		{
			unsigned m = _Ltg(0, 0).getM();
			LV = matrix<seriesTg>(1, _Qtg.GetColumn());
			R = matrix<gNg>(_Qtg.GetColumn(), 1);			
			for (unsigned i = 0; i < R.GetRow(); i++)
				R(i, 0) = gNg(0, m, _Rtg(i, 0).getB(), _Rtg(i, 0).getNr());			
			for (unsigned j = 0; j < _Qtg.GetColumn(); j++)
			{
				seriesTg sum;
				for (unsigned i = 0; i < _Qtg.GetRow(); i++){
					seriesTg sTmp = scale(_Qtg(i, j), m);
					if(sTmp.isRightForm())
						sum = sum + seriesTg(sTmp.getP()*Tg(0, i),sTmp.getQ()*Tg(0,i),sTmp.getR());
					else
						sum = sum + seriesTg(sTmp.getP()*Tg(0, i), sTmp.getQ()*Tg(0, i), sTmp.getR(),false);
				}
				LV(0, j) = sum;
			}
		}
	}
		
	void seriesET::toRVectorTg(matrix<seriesTg> & RV, matrix<gNg> & L) const
	{
		if (!isEpsilon() && !isTop())
		{
			int b = (int)_Rtg(0, 0).getB();
			RV = matrix<seriesTg>(_Qtg.GetRow(),1);
			L = matrix<gNg>(1,_Qtg.GetRow());
			for (unsigned i = 0; i < L.GetColumn(); i++)
				L(0,i) = gNg(i,_Ltg(0,i).getM(),b,0);
			for (int i = 0; i < _Qtg.GetRow(); i++)
			{
				seriesTg sum;
				for (unsigned j = 0; j < _Qtg.GetColumn(); j++) {
					//cout << "b" << b <<  endl;
					//cout << "b-1-j=" << b - 1 - j;
					//cout << "Qtg_ij=" << _Qtg(i, j) << endl;
					seriesTg sTmp = scale(_Qtg(i, j), b);
					//cout << "scale " << sTmp << endl;
					if (sTmp.isRightForm())
					{
						auto x = seriesTg(Tg(0, b - 1 - j)*sTmp.getP(), Tg(0, b - 1 - j)*sTmp.getQ(), sTmp.getR());
						//cout << x << endl;
						sum = sum + x;
					}
					else
					{
						auto x = seriesTg(Tg(0, b - 1 - j)*sTmp.getP(), Tg(0, b - 1 - j)*sTmp.getQ(), sTmp.getR(),false);
						//cout << x << endl;
						sum = sum + x;
					}

				}
				RV(i,0) = sum;
			}
		}
	}

	void seriesET::toLVectorEd(matrix<seriesEd> & LV, matrix<dDd> & R) const
	{
		if (!isEpsilon() && !isTop())
		{
			unsigned v = _Led(0, 0).getV();
			LV = matrix<seriesEd>(1, _Qed.GetColumn());
			R = matrix<dDd>(_Qed.GetColumn(), 1);
			for (unsigned i = 0; i < R.GetRow(); i++)
				R(i, 0) = dDd(0, v, _Red(i, 0).getW(), _Red(i, 0).getTr());
			for (unsigned j = 0; j < _Qed.GetColumn(); j++)
			{
				seriesEd sum;
				for (int i = 0; i < _Qed.GetRow(); i++) {
					seriesEd sTmp = scale(_Qed(i, j), v);
					if (sTmp.isRightForm())
						sum = sum + seriesEd(sTmp.getP()*Ed(0, -i), sTmp.getQ()*Ed(0, -i), sTmp.getR());
					else
						sum = sum + seriesEd(sTmp.getP()*Ed(0, -i), sTmp.getQ()*Ed(0, -i), sTmp.getR(), false);
				}
				LV(0, j) = sum;
			}
		}
	}

	void seriesET::toRVectorEd(matrix<seriesEd> & RV, matrix<dDd> & L) const
	{
		if (!isEpsilon() && !isTop())
		{
			int w = (int)_Red(0, 0).getW();
			RV = matrix<seriesEd>(_Qed.GetRow(), 1);
			L = matrix<dDd>(1, _Qed.GetRow());
			for (unsigned i = 0; i < L.GetColumn(); i++)
				L(0, i) = dDd(-i, _Led(0, i).getV(), w, 0);
			for (int i = 0; i < _Qed.GetRow(); i++)
			{
				seriesEd sum;
				for (unsigned j = 0; j < _Qed.GetColumn(); j++) {
					//cout << "b" << b <<  endl;
					//cout << "b-1-j=" << b - 1 - j;
					//cout << "Qtg_ij=" << _Qtg(i, j) << endl;
					seriesEd sTmp = scale(_Qed(i, j), w);
					//cout << "scale " << sTmp << endl;
					if (sTmp.isRightForm())
					{
						auto x = seriesEd(Ed(0, 1 - w + j)*sTmp.getP(), Ed(0, 1 - w + j)*sTmp.getQ(), sTmp.getR());
						//cout << x << endl;
						sum = sum + x;
					}
					else
					{
						auto x = seriesEd(Ed(0, 1 - w + j)*sTmp.getP(), Ed(0, 1 - w + j)*sTmp.getQ(), sTmp.getR());
						//cout << x << endl;
						sum = sum + x;
					}

				}
				RV(i, 0) = sum;
			}
		}
	}

	string seriesET::toString(unsigned f ) const
	{
		if (this->isEpsilon()) return "eps";
		else
		{
			switch (f)
			{
			case 0:
				if (_QEdForm)
				{
					stringstream ss;
					dDd::setCanonForm(0);
					if (_Led.GetColumn() != 1)
						ss << "L=(" << _Led(0, 0) << " .. " << _Led(0, _Led.GetColumn() - 1) << ")" << endl;
					if (_Qed.GetRow() == _Qed.GetColumn() && _Qed.GetRow() == 1)
					{
						ss << _Qed(0, 0) << endl;
					}
					else
					{
						ss << "{" << _Qed << "}"<< endl;
					}
					dDd::setCanonForm(2);
					if (_Red.GetRow() != 1)
						ss << "R=(" << _Red(0, 0) << " .. " << _Red(_Red.GetRow() - 1, 0) << ")'" << endl;
					return ss.str();
				}
				else
				{
					stringstream ss;
					gNg::setCanonForm(0);
					if (_Ltg.GetColumn() != 1)
						ss << "L=(" << _Ltg(0, 0) << " .. " << _Ltg(0, _Ltg.GetColumn() - 1) << ")" << endl;
					if (_Qtg.GetRow() == _Qtg.GetColumn() && _Qtg.GetRow() == 1)
					{
						ss << _Qtg(0, 0) << endl;
					}
					else ss << "{" << _Qtg << "}" << endl;
					gNg::setCanonForm(2);
					if (_Rtg.GetRow() != 1)
						ss << "R=(" << _Rtg(0, 0) << " .. " << _Rtg(_Rtg.GetRow() - 1, 0) << ")'" << endl;
					return ss.str();
				}
				break;
			case 1:			
				// LVxR out
				if (!_QEdForm)
				{
					gNg::setCanonForm(2);
					stringstream ss;
					matrix<seriesTg> LV;
					matrix<gNg> R;
					toLVectorTg(LV, R);					
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < LV.GetColumn(); i++)
					{						
						if (!LV(0, i).isEpsilon())
						{
							LV(0, i).toRight();
							if (dejaUneSortie) ss << "+";
							ss << "(" << LV(0, i) << ")." << R(i, 0);
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}
				else
				{
					dDd::setCanonForm(2);
					stringstream ss;
					matrix<seriesEd> LV;
					matrix<dDd> R;
					toLVectorEd(LV, R);
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < LV.GetColumn(); i++)
					{
						if (!LV(0, i).isEpsilon())
						{
							LV(0, i).toRight();
							if (dejaUneSortie) ss << "+";
							ss << "(" << LV(0, i) << ")." << R(i, 0);
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}
				break;
			case 2:
				// LxR out
				if (!_QEdForm)
				{
					gNg::setCanonForm(0);
					stringstream ss;
					matrix<seriesTg> RV;
					matrix<gNg> L;
					toRVectorTg(RV, L);					
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < RV.GetRow(); i++)
					{
						if (!RV(i,0).isEpsilon())
						{
							RV(i,0).toLeft();
							if (dejaUneSortie) ss << "+";
							ss <<L(0,i) << ".(" << RV(i,0) << ")";
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}
				else
				{
					dDd::setCanonForm(0);
					stringstream ss;
					matrix<seriesEd> RV;
					matrix<dDd> L;
					toRVectorEd(RV, L);				
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < RV.GetRow(); i++)
					{
						if (!RV(i, 0).isEpsilon())
						{
							RV(i, 0).toLeft();
							if (dejaUneSortie) ss << "+";
							ss << L(0, i) << ".(" << RV(i, 0) << ")";
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}				
				break;
			case 3:  
				if (!_QEdForm)
				{
					// DeltaVar (right)
					gNg::setCanonForm(2);
					stringstream ss;
					matrix<seriesTg> LV;
					matrix<gNg> R;
					toLVectorTg(LV, R);
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < LV.GetColumn(); i++)
					{
						if (!LV(0, i).isEpsilon())
						{
							LV(0, i).toRight();
							if (dejaUneSortie) ss << "+";
							ss << "(" << LV(0, i).toStringAsDeltaVar() << ")." << R(i, 0);
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}
				else
				{
					dDd::setCanonForm(2);
					stringstream ss;
					matrix<seriesEd> LV;
					matrix<dDd> R;
					toLVectorEd(LV, R);
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < LV.GetColumn(); i++)
					{
						if (!LV(0, i).isEpsilon())
						{
							LV(0, i).toRight();
							if (dejaUneSortie) ss << "+";
							ss << "(" << LV(0, i).toStringAsMuVar() << ")." << R(i, 0);
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}
				break;				
			default :
				// LxR out
				if (!_QEdForm)
				{
					gNg::setCanonForm(0);
					stringstream ss;
					matrix<seriesTg> RV;
					matrix<gNg> L;
					toRVectorTg(RV, L);
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < RV.GetRow(); i++)
					{
						if (!RV(i, 0).isEpsilon())
						{
							RV(i, 0).toLeft();
							if (dejaUneSortie) ss << "+";
							ss << L(0, i) << ".(" << RV(i, 0).toStringAsDeltaVar() << ")";
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}
				else
				{
					dDd::setCanonForm(0);
					stringstream ss;
					matrix<seriesEd> RV;
					matrix<dDd> L;
					toRVectorEd(RV, L);
					bool dejaUneSortie = false;
					for (unsigned i = 0; i < RV.GetRow(); i++)
					{
						if (!RV(i, 0).isEpsilon())
						{
							RV(i, 0).toLeft();
							if (dejaUneSortie) ss << "+";
							ss << L(0, i) << ".(" << RV(i, 0).toStringAsMuVar() << ")";
							dejaUneSortie = true;
						}
					}
					return ss.str();
				}
				break;							
			}
		}
	}

	seriesET seriesET::toCausal(const seriesET & s)
	{
		seriesET cS(s);
		cS.toCoreTg();
		matrix<seriesTg> Qc =cS._Qtg;			
		for (unsigned i = 0; i <Qc.GetRow(); i++)
		{
			for (unsigned j = 0; j < Qc.GetColumn(); j++)
			{
				Qc(i, j) = seriesTg::toCausal(Qc(i,j));
			}
		}
		return seriesET(cS._Ltg, Qc, cS._Rtg);
	}

	//////////////////////////////////////////////////////////////////
	//global operations for matrix<seriesET> 
	seriesET star(const seriesET & s){	return s.star();	}

	seriesET oplus(const seriesET & sa, const seriesET & sb){
		return sa.oplus(sb);
	}

	seriesET otimes(const seriesET & sa, const seriesET & sb){
		return sa.otimes(sb);
	}

	seriesET inf(const seriesET & sa, const seriesET & sb) {
		return sa.inf(sb);
	}

	seriesET lfrac(const seriesET & sa, const seriesET & sb) {
		return sa.lfrac(sb);
	}

	seriesET rfrac(const seriesET & sa, const seriesET & sb) {
		return sa.rfrac(sb);
	}



}//end namespace etvo
