// mainDocETVO.cpp
// Examples illustrating the ETVO documentation

// mainDocETVO.cpp
// Simple C++ examples 

/* Illustrates the examples presented in 

 "(EVENT|TIME)-VARIANT OPERATORS " 
 A C++  toolbox to handle series for event-variant/time-variant (max,+) systems
  B.Cottenceau L.Hardouin J.Trunk
  Tech. Report LARIS 

*/

#include "../etvo.h"
#include<iostream>
using namespace std;
using namespace etvo;

	void Example1();
	void Example2();
	void Example3();
	void Example4();
	void Example5();
	void Example6();
	void Example7();
	void Example8();
	void Example9();
	void Example10();
	void Example11();
	void Example12();
	void Example13();
	void Example14();	
	void Example15();	
	void Example16();	
	void Example17();	
	void Example18();	
	void Example19();	
	void Example20();	
	void Example21();	
	
int main()
{
	
	Example1();
	Example2();
	Example3();
	Example4();	
	Example5();	
	Example6();	
	Example7();	
	Example8();	
	Example9();	
	Example10();	
	Example11();	
	Example12();
	Example13();
	Example14();
	Example15();
	Example16();
	Example17();	
	Example18();	
	Example19();
	Example20();
	
}

void Example20()
{
	cout << endl << "Example Listing 5.13" << endl;
	series sh1(poly(),gd(0,2),gd(3,2));
	series sh2(poly(),gd(0,4),gd(3,4));
	series sh3(poly(),gd(0,3),gd(2,3));
	series sh4(sh2);
	
	seriesEd eh1=seriesEd::toSeriesEd(sh1);
	seriesEd eh2=seriesEd::toSeriesEd(sh2);
	seriesEd eh3=seriesEd::toSeriesEd(sh3);
	seriesEd eh4=seriesEd::toSeriesEd(sh4);
	
	seriesET h1(eh1);
	seriesET h2(eh2);
	seriesET h3(eh3);
	seriesET h4(eh4);
	
	cout << "eh1=" << eh1 << endl;
	cout << "eh2=" << eh2 << endl;
	cout << "eh3=" << eh3 << endl;
	cout << "eh4=" << eh4 << endl;
	
	seriesET G = etb({0,1})*h3*etd(1)*etd({0,0,2,1})*etd(1)*h2*etm({0,1});
	G=G+ (etb({1,0})*h4*etd(1)*etd({2,1,0,0})*etd(1)*h1*etm({1,0}));
	G=etd(1)*G*etd(2);
	G.toCoreEd();
	cout << "G=" << G << endl;
	G.toCoreTg();
	cout << "G=" << G << endl;
	
	
}

void Example1()
{

	cout << endl << "Example Listing 3.1" << endl;
 // Example 1 Listing 3.1 TEG with MinMaxGD
 // series corresponds to MinMaxGD series
	matrix<series> A(4, 4), B(4, 1), C(1, 4);
	B(0, 0) = gd(0, 0); 	 // \gamma^0\delta^0=e
	C(0, 1) = gd(0, 0);
	C(0, 3) = gd(1, 0);	// \gamma^1
	A(0, 3) = gd(2, 0);
	A(1, 0) = gd(0, 2);	// \delta^2
	A(2, 1) = gd(0, 0);
	A(3, 2) = gd(0, 3);

	matrix<series>  H = C * A.star() * B;
	cout << H(0, 0) << endl;
	// output : (g0.d2+g1.d5).[g2.d5]*
}

void Example2()
{
	// Listing 3.3
	cout << endl << "Example Listing 3.3" << endl;
	matrix<seriesEd> A(4,4), B(4,1), C(1,4);
	B(0, 0) = eb(2); //b2
	C(0, 1) = eb(2);
	C(0, 3) = eg(1); //g1
	A(0, 3) = eb(3)*eg(5)*em(2); //b3.g5.m2
	A(1, 0) = em(3)*ed(2); //m3.d2
	A(2, 1) = eg(0); //g0=e
	A(3, 2) = eb(2)*ed(5); //b2.d5
	matrix<seriesEd> H=C*A.star()*B;
	H(0, 0).toRight();
	cout << H(0,0) << endl;
	//output=((m3.b4.g2+g1.m3.b4).d2+(g1.m3.b4.g2+g2.m3.b4).d7).[g2.d7]*
}

void Example3()
{
	// Listing 3.5
	cout << endl << "Example Listing 3.5" << endl;
	matrix<seriesTg> A(4,4), B(4,1), C(1, 4);
	B(0, 0) = tD(3); //D3=d<0,2,1>
	C(0, 1) = td({1,0}); //d<1,0>
	C(0, 3) = tg(1); //g1
	A(0, 3) = tg(2);
	A(1, 0) = td(1); //d1
	A(2, 1) = tg(0);
	A(3, 2) = td({2,3,2});
	matrix<seriesTg> G=C*A.star()*B;
	G(0,0).toRight();
	cout << G(0,0).toStringAsDeltaVar() << endl;
	//output=((d<1,4,3,2,3,2>.g0))+(d<4,6,5>.g1+d<5,8,7,6,7,6>.g2).[d3.g2]*
}

void Example4()
{
	// Listing 3.7
	cout << endl << "Example Listing 3.7" << endl;
	seriesEd mvar,bvar;
	mvar=em({1,3,2}); // m<1,3,2>
	bvar=eb({1,2}); // b<1,2>
	cout << "mvar=" << mvar << endl;
	cout << "bvar=" << bvar << endl;
	//output mvar=((m6.b3.g2+g1.m6.b3.g1+g4.m6.b3).d0)
	//output bvar=((m2.b3.g2+g1.m2.b3).d0)
}


void Example5()
{
	// Listing 3.9
	cout << endl << "Example Listing 3.9" << endl;
	matrix<seriesEd> A(2,2), B(2,2),C(1,2),H(1,2);
	A(0,1)=eg(2);
	A(1,0)=ed(1)*em({1,0,2});
	B(0,0)=ed(4)*em({2,3});
	B(0,1)=eb({2,1,1})*eg(1);
	C(0,1)=eg(0);
	H=C*A.star()*B;
	H(0,0).toRight();
	H(0,1).toRight();
	cout << H << endl;	
}

void Example6()
{
	// Listing 3.11
	cout << endl << "Example Listing 3.11" << endl;
	series h1(poly(),poly(gd(0,4)),gd(2,3));
	seriesEd eh1=seriesEd::toSeriesEd(h1);
	
	series h2=series(poly(),gd(0,4),gd(4,4))*series(poly(),gd(0,1),gd(2,1));
	seriesEd eh2=seriesEd::toSeriesEd(h2);
	
	seriesEd eG=(eb({1,0,1,0,0})*eh1*em({1,0,1,0,0}))+(eb({0,1,0,1,1})*eh2*em({0,1,0,1,1}));
	cout << "eh1=" << eh1 << endl;
	cout << "eh2=" << eh2 << endl;
	cout << "eG=" << eG << endl;
	cout << "eG=" << eG.toStringAsMuVar() << endl;	
}

void Example7()
{
	// Listing 3.12
	cout << endl << "Example Listing 3.12" << endl;
	series h1(poly(),poly(gd(0,2)),gd(1,1));
	seriesTg th1=td({0,2,1})*seriesTg::toSeriesTg(h1);
	series h2(poly(),poly(gd(0,5)),gd(2,4));
	seriesTg th2=seriesTg::toSeriesTg(h2);
	cout << "th1=" << th1 << endl;
	cout << "th2=" << th2 << endl;
	
	seriesTg tG=th2*tv(2)*tw(3)*th1;
	cout << "tG=" << tG << endl;
	
}

void Example8()
{
	// Listing 4.1
	cout << endl << "Example Listing 4.1" << endl;
	series H;
	H=series(gd(0,2)+gd(1,5))*series(gd(2,5)).star();
	series U=gd(0,4)+gd(1,8)+gd(5,11)+gd(6,10000);
	cout << "H=" << H << endl;
	series Y=H*U;
	cout << "Y=" << Y << endl;
	/* output H=(g0.d2+g1.d5).[g2.d5]*
	output Y=g0.d6+g1.d10+g2.d13+g3.d15+g4.d18+g5.d20
	+(g6.d10002+g7.d10005).[g2.d5]*
	= g0.d6+g1.d10+g2.d13+g3.d15+g4.d18
	+g5.d20+g6.d10002+g7.d10005 + ....
	*/
	
}

void Example9()
{
	// Listing 4.3
	cout << endl << "Example Listing 4.3" << endl;
	seriesEd H;
	H=Ed(0,3,4,2,2)+Ed(1,3,4,0,2)+Ed(1,3,4,2,7)+Ed(2,3,4,0,7);
	H=H*seriesEd(Ed(0,1,1,2,7)).star();
	series U=gd(0,4)+gd(1,8)+gd(5,11)+gd(6,13)+gd(7,15)+gd(8,16)+gd(9,100000);
	seriesEd Um=seriesEd::toSeriesEd(U);
	seriesEd Y=H*Um;
	series y = Y.toSeries();
	cout << "U=" <<U << endl;
	cout << "H=" <<H << endl;
	cout << "y=" <<y << endl;
	/* output
	H=((m3.b4.g2+g1.m3.b4).d2+(g1.m3.b4.g2+g2.m3.b4).d7).[g2.d7]*
	y=g0.d10+g1.d17+g2.d22+g3.d24+g4.d31+g5.d36+(g6.d100002+ ...
	*/	
}

void Example10()
{
	// Listing 4.5
	cout << endl << "Example Listing 4.5" << endl;
	seriesTg H;
	H=(td({4,6,5})*tg(1)+td({5,8,7,6,7,6})*tg(2))*(seriesTg(Tg(3,2)).star());
	H=H+td({1,4,3,2,3,2});
	series u=gd(0,4)+gd(1,8)+gd(5,11)+gd(6,13)+gd(7,15)+gd(8,16)+gd(9,100000);
	seriesTg Um=seriesTg::toSeriesTg(u);
	seriesTg Y=H*Um;
	series y=Y.toSeries();
	H.toRight();
	cout << "H=" << H << endl;
	cout << "H=" << H.toStringAsDeltaVar() << endl;
	cout << "u=" << u << endl;
	cout << "y=" << y << endl;	
}

void Example11()
{
	// Listing 4.7
	cout << endl << "Example Listing 4.7" << endl;
	series H;
	H=series(gd(0,2)+gd(1,5))*series(gd(2,5)).star();
	series z(poly::Epsilon(),gd(6,10000),gd(2,5));
	z=z+gd(0,10)+gd(3,13)+gd(5,19);
	series uopt=z.frac(H);
	series yopt=H*uopt;
	cout << "uopt=" << uopt << endl;
	cout << "yopt=" << yopt << endl;
	//uopt=g0.d1+g1.d3+g2.d6+g3.d8+g4.d11+g5.d17+(g6.d9995+...
	//yopt=g0.d3+g1.d6+g2.d8+g3.d11+g4.d13+g5.d19+(g6.d9997+...
}

void Example12()
{
	// Listing 4.9
	cout << endl << "Example Listing 4.9" << endl;
	series H;
	H=series(gd(0,2)+gd(1,5))*series(gd(2,5)).star();
	series Popt=H.frac(H);
	series Fopt = (Popt.frac(H)).prcaus();
	series C1=H*Popt;
	series C2=H*(Fopt*H).star();
	cout << "H=" << H << endl;
	cout << "Popt=" << Popt << endl;
	cout << "Fopt=" << Fopt << endl;		
	cout << "C1=" << C1 << endl;
	cout << "C2=" << C2 << endl;
	
	
}

void Example13()
{
	// Listing 4.10
	cout << endl << "Example Listing 4.10" << endl;
	matrix<seriesEd> A(4,4), B(4,1), C(1,4);
	B(0, 0) = eb(2); //b2
	C(0, 1) = eb(2);
	C(0, 3) = eg(1); //g1
	A(0, 3) = eb(3)*eg(5)*em(2); //b3.g5.m2
	A(1, 0) = em(3)*ed(2); //m3.d2
	A(2, 1) = eg(0); //g0=e
	A(3, 2) = eb(2)*ed(5); //b2.d5
	matrix<seriesEd> H=C*A.star()*B;
	
	seriesEd eH=H(0, 0);
	eH.toRight();
	cout << "H=" << eH << endl;
	//output=((m3.b4.g2+g1.m3.b4).d2+(g1.m3.b4.g2+g2.m3.b4).d7).[g2.d7]*

	seriesEd Popt=eH.lfrac(eH);
	seriesEd Fopt=seriesEd::toCausal(Popt.rfrac(eH));
	
	cout << "Popt=" << Popt << endl;
	cout << "Fopt=" << Fopt << endl;
	
	seriesEd CL1=eH*Popt;
	seriesEd CL2=eH*(Fopt*eH).star();
	CL1.toRight();	
	CL2.toRight();
	
	cout << "CL1=" << CL1 << endl;
	cout << "CL2=" << CL2 << endl;	
}

void Example14()
{
	//Listing 5.1
	cout << endl << "Example Listing 5.1" << endl;	
	series H;
	series M1(poly::Epsilon(),gd(0,6),gd(5,6));
	series M2(poly::Epsilon(),gd(0,5),gd(4,5));
	H=series(gd(0,4))*M2*gd(0,1)*M1*gd(0,4);
	cout << "H=" << H << endl;	
}

void Example15()
{
	//Listing 5.3
	cout << endl << "Example Listing 5.3" << endl;	
	series H, F;
	series M1(poly::Epsilon(),gd(0,6),gd(5,6));
	series M2(poly::Epsilon(),gd(0,5),gd(4,5));
	H=series(gd(0,4))*M2*gd(0,1)*M1*gd(0,4);
	// F=H\H/H
	F=H.frac(H).frac(H);
	// causal projection
	F=F.prcaus();
	cout << "F=" << F << endl;
	series Fs=series(gd(19,3));
	cout << "Fs=" << Fs << endl;
	cout << "H.(Fs.H)*=" << H*(Fs*H).star() << endl;
}

void Example16()
{
	cout << endl << "Example Listing 5.5" << endl;
	seriesEd G;
	series H1(poly::Epsilon(),gd(0,2),gd(1,2));
	series H2(poly::Epsilon(),gd(0,1),gd(2,1));
	G=ed(2)*em(3)*seriesEd::toSeriesEd(H1)*eb(2)*ed(2);
	G=G+eb(2)*eg(1)*ed(1)*seriesEd::toSeriesEd(H2)*ed(2)*em(3)*ed(1);
	G=ed(1)*G*ed(2);
	cout << "G=" << G << endl;
}

void Example17()
{
	cout << endl << "Example Listing 5.7" << endl;
	seriesEd G;
	series H1(poly::Epsilon(),gd(0,1),gd(2,1));
	series H2(poly::Epsilon(),gd(0,3),gd(2,3));
	series H3(poly::Epsilon(),gd(0,4),gd(3,4));
	G=eb({1,0,0,1,0})*seriesEd::toSeriesEd(H2)*ed(2)*em({0,1,0,1,0});
	G=G+eb({0,1,1,0,1})*seriesEd::toSeriesEd(H3)*ed(2)*em({1,0,1,0,1});
	G=ed(1)*G*ed(3)*seriesEd::toSeriesEd(H1)*ed(3);
	cout << "G=" << G << endl;
	cout << "G=" << G.toStringAsMuVar() << endl;
}

void Example18()
{
	cout << endl << "Example Listing 5.9" << endl;
	seriesEd G;
	series H1(poly::Epsilon(),gd(0,6),gd(5,6));
	series H2(poly::Epsilon(),gd(0,5),gd(4,5));
	series H3(poly::Epsilon(),gd(0,7),gd(1,7));
	series H4(poly::Epsilon(),gd(0,8),gd(2,8));
	G=eb({0,1})*ed(1)*em(2)*seriesEd::toSeriesEd(H3)*ed(1)*em({0,1,0});
	G=G+eb({1,0})*ed(1)*seriesEd::toSeriesEd(H4)*ed(1)*em({1,0,1});
	G=ed(1)*G*eb(2)*ed(3)*seriesEd::toSeriesEd(H2);
	G=G*ed(1)*seriesEd::toSeriesEd(H1)*ed(1);
	cout << "G=" << G << endl;
	cout << "G=" << G.toStringAsMuVar() << endl;
}

void Example19()
{
	cout << endl << "Example Listing 5.11" << endl;
	seriesTg G;
	series H1(poly::Epsilon(),gd(0,6),gd(5,6));
	series H2(poly::Epsilon(),gd(0,5),gd(4,5));
	G=td(3)*seriesTg::toSeriesTg(H2)*td(1);
	G=G*td({2,1,0,0,1})*td(1)*seriesTg::toSeriesTg(H1)*td(1);
	cout << "G=" << G << endl;
	cout << "G=" << G.toStringAsDeltaVar() << endl;
}
