//mainIEEETASE17.cpp
/* Examples from
Cottenceau, B., Hardouin, L., & Trunk, J. (2017). 
Weight-balanced timed event graphs to model periodic phenomena in manufacturing systems. 
IEEE Transactions on Automation Science and Engineering, 14(4), 1731-1742.
*/
#include <iostream>
#include <exception>
#include "../etvo.h"

using namespace std;
using namespace etvo;

int main()
{ 
	// Example 6 IEEE TASE
	seriesEd op1 = em({ 1,3,2 });
	seriesEd op2 = em({ 1,0,2 });

	cout << "op1=" << op1 << endl;
	cout << "op2=" << op2 << endl;

	//Example 7
	seriesEd op3 = eb({ 1,2,0 });
	cout << "op3=" << op3 << endl;

	seriesEd op4 = eb({ 1,2 });
	cout << "op4=" << op4 << endl;
	cout << "op4=" << op4.toStringAsMuVar() << endl;

	//Example 8 IEEE TASE
	cout << "m<2,3> =" << em({ 2,3 }) << endl;
	cout << "b<2,1,1>.g1 =" << eb({ 2,1,1 })*eg(1) << endl;
	cout << "m<1,0,2> =" << em({ 1,0,2 }) << endl;

	//Example Fig11 - Fig12
	matrix<seriesEd> A(2, 2), B(2, 2), C(1, 2);
	B(0, 0) = ed(4)*em({ 2,3 });
	B(0, 1) = eb({ 2,1,1 })*eg(1);

	C(0, 1) = eg(0);

	A(1, 0) = ed(1)*em({ 1,0,2 });
	A(0, 1) = eg(2);

	cout << "A=" << A << endl;
	cout << "B=" << B << endl;
	cout << "C=" << C << endl;

	matrix<seriesEd> As = A.star();

	matrix<seriesEd>  H = (C * As*B);	
	seriesEd h1 = H(0, 0);
	seriesEd h2 = H(0, 1);
	h1.toLeft();
	cout << "h1=" << h1 << endl;
	cout << "h1=" << h1.toStringAsMuVar() << endl;

	h2.toLeft();
	cout << "h2=" << h2 << endl;
	cout << "h2=" << h2.toStringAsMuVar() << endl;


	// Example Fig17
	seriesEd H1 = ed(4)*(eg(2)*ed(3)).star();
	seriesEd H2 = (ed(5)+eg(2)*ed(6))*(eg(4)*ed(4)).star();
	cout << H1 << endl;
	cout << H2 << endl;
	seriesEd Ga = eb({ 1,0,1,0,0 })*H1*em({ 1,0,1,0,0 });
	Ga = Ga+ eb({ 0,1,0,1,1 })*H2*em({ 0,1,0,1,1 });

	cout << "Ga=" <<Ga.toStringAsMuVar() << endl;

	seriesEd Fa = Ga.lfrac(Ga).rfrac(Ga);
	Fa = seriesEd::toCausal(Fa);
	cout << "Fa=" << Fa << endl;
	
	seriesEd CLA = Ga * (eg(10)*Ga).star();
	cout << "CLA=" << CLA.toStringAsMuVar() << endl;
	cout << "CLA==Ga :" << ((CLA == Ga)?"true":"false") << endl;


	// Example Fig20
	A = matrix<seriesEd>(3, 3);
	B = matrix<seriesEd>(3, 1);
	C = matrix<seriesEd>(1, 3);

	A(1, 0) = eb({ 1,0,0 })*ed(3)*em({ 1,0,0 });
	A(1, 0) = A(1,0) + eb({ 0,1,0 })*ed(4)*em({ 0,1,0 });
	A(1, 0) = A(1, 0) + eb({ 0,0,1 })*ed(6)*em({ 0,0,1});
	A(0, 1) = eg(3);
	A(2, 1) = eb({ 1,0 })*ed(2)*em({ 1,0 });
	A(2, 1) = A(2, 1) + eb({ 0,1 })*ed(5)*em({ 0,1 });
	
	B(0, 0) = eg(0);
	C(0, 2) = eg(0);
	
	cout << "A=" << A << endl;
	cout << "B=" << B << endl;
	cout << "C=" << C << endl;

	matrix<seriesEd>  Gb = (C * A.star()*B);

	cout << "Gb=" << Gb(0, 0).toStringAsMuVar() << endl;
  
}

 
  
  


 
