#include <iostream>
#include <exception>
#include <ctime>
#include <ratio>
#include <chrono>
#include "../etvo.h"
#include "testsEtvo.h"
using namespace std;
using namespace etvo;
using namespace test;
using namespace std::chrono;

int main(int argc, char *argv[])
{			

	bool testUnit = false;
	bool testSpecific = false;
	bool testRegular = false;

	unsigned nIter = 3;
	unsigned nLoops = 1;
	unsigned gain = 3;
	unsigned nTerms = 3;
	unsigned perCentEps = 10;
	unsigned perCentPoly = 20;
	bool fper = false;

	if (argc < 2)
	{
		cout << "Needs arguments: -u (unit) -s (specific) -r (regular) " << endl;
		cout << "\t  -i10 (10 iter) -l5 (5 loops) -g3 (gain 3) " << endl;
		cout << "\t  -t8 (8 terms) -e20 (20% eps) -p30 (30% poly) " << endl;
		return 1;
	}
	try {
		for (int i = 1; i < argc; i++)
		{
			string option(argv[i]);
			if (option == "-u")	testUnit = true;			
			if (option == "-r") testRegular = true;
			if (option == "-s") testSpecific = true;
			if (option == "-fper") fper = true;

			if (option.substr(0, 2) == "-i") nIter = (unsigned)stoi(option.substr(2));
			if (option.substr(0, 2) == "-l") nLoops = (unsigned)stoi(option.substr(2));
			if (option.substr(0, 2) == "-g") gain = (unsigned)stoi(option.substr(2));
			if (option.substr(0, 2) == "-t") nTerms = (unsigned)stoi(option.substr(2));			
			if (option.substr(0, 2) == "-e")
			{
				perCentEps = (unsigned)stoi(option.substr(2));
				if (perCentEps > 100) perCentEps = 100;
			}
			if (option.substr(0, 2) == "-p")
			{
				perCentPoly = (unsigned)stoi(option.substr(2));
				if (perCentPoly > 100) perCentEps = 100;
			}
		}
	}
	catch (exception ex)
	{
		cout << ex.what() << endl;
		return 1;
	}
	cout << "Test parameters :";
	if (testUnit) cout << " unit";
	if (testSpecific) cout << " specific";
	if (testRegular) cout << " regular" << endl;
	if (testRegular | testSpecific)
	{
		cout << " nIters=" << nIter;
		cout << " nLoops=" << nLoops;
		cout << " gain=" << gain;
		cout << " terms=" << nTerms << endl;
		cout << "\t %eps=" << perCentEps << "%" << endl;
		cout << "\t %poly=" << perCentPoly << "%" << endl;
	}

	Test::percentEpsInFactory = perCentEps;
	Test::percentPolyInFactory = perCentPoly;
	Test::setup(nIter,nLoops,nTerms,gain);

	if(testUnit) Test::ScenarioAllUnit();
	if(testSpecific) Test::ScenarioAllSpecific(nIter,nLoops);
	if(testRegular) Test::ScenarioAllRegular(nIter, nLoops,fper);
	return 0;
}

 
  
  


 
