#include "parser.h"
#define BOOST_SPIRIT_NO_PREDEFINED_TERMINALS

#include <boost/config/warning_disable.hpp>
#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/qi_int.hpp>
#include <boost/spirit/include/qi_char_class.hpp>
#include <iostream>
#include <string>
#include <stack>
#include<map>
#include<algorithm>
#include "../etvo/common/etvoException.h"
#include "../test/factory/randGen.h"

using namespace std;
using namespace etvo;

typedef std::map<std::string, etvo::seriesET> symtab_set;
typedef std::map<std::string, etvo::matrix<seriesET>> symtab_met;

namespace calculatorEtvo
{
  symtab_set vars_ET;  // vars in ET
  symtab_met vars_mET;  // vars in matrix<seriesET>
  std::stack<etvo::seriesET> stack_ET;
  std::stack<etvo::matrix<seriesET>> stack_mET;
  std::stack<std::string> stack_identET;
  std::stack<std::string> stack_identmET;

  std::vector<unsigned> vect_Uint;
  std::vector<int> vect_Int;
  std::vector<int> vect_IdxInt;
  std::string strIdent;

  namespace qi = boost::spirit::qi;
  namespace ascii = boost::spirit::ascii;
    
  void do_pushC(char c)
  {
	  strIdent += c;
  }
  void do_pushInt(int w)
  {
	  vect_Int.push_back(w);
  }

  void do_pushIdxInt(int w)
  {
	  vect_IdxInt.push_back(w);
  }

  void do_pushUint(unsigned w)
  {
	  vect_Uint.push_back(w);
  }

// all semantic actions are in included source files
#include "doET.cpp"
#include "domET.cpp"
  
  void do_randEd()
  {
	  int nbTerms = vect_Int.back();
	  vect_Int.pop_back();
	  int div = vect_Int.back();
	  vect_Int.pop_back();
	  int mul = vect_Int.back();
	  vect_Int.pop_back();
	  stack_ET.push(seriesET(randGen::Rand_seriesEd_fixedG(mul, div, nbTerms)));
  }

  void do_randTg()
  {
	  int nbTerms = vect_Int.back();
	  vect_Int.pop_back();
	  int div = vect_Int.back();
	  vect_Int.pop_back();
	  int mul = vect_Int.back();
	  vect_Int.pop_back();
	  stack_ET.push(seriesET(randGen::Rand_seriesTg_fixedG(mul, div, nbTerms)));
  }

  void do_randMM()
  {
	  int nbTerms = vect_Int.back();
	  vect_Int.pop_back();
	  gd slope(randGen::uni_int(1, 10), randGen::uni_int(1, 10));
	  stack_ET.push(seriesET(seriesEd(seriesEd::toSeriesEd(randGen::Rand_series(nbTerms,slope)))));
  }

  
 //------------------------------------------------------------------------------------

  ///////////////////////////////////////////////////////////////////////////////
  //  Our calculator grammar
  ///////////////////////////////////////////////////////////////////////////////
  template <typename Iterator>
  struct calculator : qi::grammar<Iterator, ascii::space_type>
  {
    calculator() : calculator::base_type(statement)
    {
      qi::int_type int_;
      qi::uint_type uint_;
      qi::alnum_type alnum_;
      qi::lexeme_type lexeme_;     
	 
	  randMM = ("randMM(" >> int_[&do_pushInt] >> ')')[&do_randMM];
	  randEd = ("randEd(" >> int_[&do_pushInt]
		  >> ',' >> int_[&do_pushInt] >> ',' >> int_[&do_pushInt] >> ')')[&do_randEd];
	  randTg = ("randTg(" >> int_[&do_pushInt]
		  >> ',' >> int_[&do_pushInt] >> ',' >> int_[&do_pushInt] >> ')')[&do_randTg];
	    epsET = (lexeme_["eps"])[&do_pushEpsET];

	  gammaET = 'g' >> int_[&do_pushGammaET];
      deltaET = 'd' >> int_[&do_pushDeltaET];
      muET = 'm' >> int_[&do_pushMuET];
	  betaET = 'b' >> int_[&do_pushBetaET];
	  veeET = 'v' >> int_[&do_pushVeeET];
	  wedgeET = 'w' >> int_[&do_pushWedgeET];
	  seqET = '<' >> uint_[&do_pushUint] >> *(',' >> uint_[&do_pushUint]) >> '>';
	  muVarET = ('m' >> seqET)[&do_pushMuVarET];
	  betaVarET = ('b' >> seqET)[&do_pushBetaVarET];
	  seqTg = '<' >> int_[&do_pushInt] >> *(',' >> int_[&do_pushInt]) >> '>';
	  deltaVarET = ('d' >> seqTg)[&do_pushDeltaVarET];

	  infET = ("inf(" >> expressionET >> ',' >> expressionET >> ')')[&do_infET];
	  lfracET = ("lfrac(" >> expressionET >> ',' >> expressionET >> ')')[&do_lfracET];
	  rfracET = ("rfrac(" >> expressionET >> ',' >> expressionET >> ')')[&do_rfracET];
	  prcausET = ("pr+(" >> expressionET >> ')')[&do_prcausET];
	  rightET = ("right(" >> expressionET >> ')')[&do_rightET];
	  leftET = ("left(" >> expressionET >> ')')[&do_leftET];
	  equalET = ("?=(" >> identET[&do_pushVarET] >> "," >> identET[&do_pushVarET] >> ')')[&do_equalET];
	  identET = lexeme_["s" >> +(alnum_[&do_pushC])];
	  assignET = (identET[&do_pushIdentET] >> '=' >> expressionET)[&do_assignET];
	  
	  factorET = polyET | groupET | kleeneET 
		  | entrymET
		  | identET[&do_pushVarET] | infET | lfracET | rfracET | prcausET 
		  | rightET | leftET|toCoreEdET|toCoreTgET
		  | randEd | randMM | randTg;
	  kleeneET = ('[' >> expressionET >> "]*")[&do_kleeneET];
	  groupET = '(' >> expressionET >> ')';
	  termET = factorET >> *('.' >> factorET[&do_otimesET]);
	  expressionET = termET >> *('+' >> termET[&do_oplusET]);
	  polyET = gammaET | betaET | muET | muVarET |veeET|wedgeET
		  | betaVarET|deltaVarET | epsET | deltaET;

	  /////////////////////////////////////////////////////////////
      //matrix      
        
	  identmET = lexeme_["S" >> +(alnum_[&do_pushC])];
	  assignmET = (identmET[&do_pushIdentmET] >> '=' >> expressionmET)[&do_assignmET];

	  epsMatET = ("eps(" >> int_[&do_pushInt] >> ',' >> int_[&do_pushInt] >> ')')[&do_pushMatET];	
	  factormET = epsMatET | groupmET |kleenemET |  identmET[&do_pushVarmET] 
		  |infmET |lfracmET |rfracmET|prcausmET;
	  kleenemET = ('[' >> expressionmET >> "]*")[&do_kleenemET];
	  groupmET = '(' >> expressionmET >> ')';
	  termmET = factormET >> *('.' >> factormET[&do_otimesmET]);
	  expressionmET = termmET >> *('+' >> termmET[&do_oplusmET]);	 
	  assignEntrymET = (identmET[&do_pushIdentmET] >> '(' >> int_[&do_pushIdxInt] >> ',' 
		  >> int_[&do_pushIdxInt] >> ")=" >>expressionET)[&do_assignEntrymET];
	  entrymET = (identmET[&do_pushIdentmET] >> '(' >> int_[&do_pushInt] 
		  >> ',' >> int_[&do_pushInt] >> ")")[&do_pushEntrymET];
	  infmET = ("inf(" >> expressionmET >> ',' >> expressionmET >> ')')[&do_infmET];
	  lfracmET = ("lfrac(" >> expressionmET >> ',' >> expressionmET >> ')')[&do_lfracmET];
	  rfracmET = ("rfrac(" >> expressionmET >> ',' >> expressionmET >> ')')[&do_rfracmET];
	  prcausmET = ("pr+(" >> expressionmET >> ')')[&do_prcausmET];
	  	 
	  asRight = ("asRight(" >> expressionET >> ')')[&do_asRightET];
	  asLeft = ("asLeft(" >> expressionET >> ')')[&do_asLeftET];
	  toCoreEdET = ("toCoreEd(" >> expressionET >> ')')[&do_toCoreEdET];
	  toCoreTgET = ("toCoreTg(" >> expressionET >> ')')[&do_toCoreTgET];
	  
	  asMuVarR = ("asMuVarR(" >> expressionET >> ')')[&do_asMuVarRET];
	  asMuVarL = ("asMuVarL(" >> expressionET >> ')')[&do_asMuVarLET];
	  asDeltaVarR = ("asDeltaVarR(" >> expressionET >> ')')[&do_asDeltaVarRET];
	  asDeltaVarL = ("asDeltaVarL(" >> expressionET >> ')')[&do_asDeltaVarLET];
	   
	  	  
	  statement = equalET | assignET | identET[&do_printVarET]
		  | asRight| asLeft|asMuVarR|asMuVarL|asDeltaVarR|asDeltaVarL
		  | assignmET | assignEntrymET		  
		  | identmET[&do_printVarmET];
    }

	qi::rule<Iterator, ascii::space_type> seqTg,randEd,randTg,randMM;
	
	qi::rule<Iterator, ascii::space_type> infmET,lfracmET,rfracmET,identmET,
					assignmET,assignEntrymET, expressionmET,epsMatET,factormET,
					kleenemET,groupmET,termmET,entrymET,prcausmET;	

    qi::rule<Iterator, ascii::space_type> statement,expressionET,asRight,asLeft,
		asMuVarR,asMuVarL,asDeltaVarR,asDeltaVarL,
		infET,lfracET,rfracET,prcausET,rightET,leftET,toCoreEdET,toCoreTgET, 
      factorET, groupET, termET, polyET, gammaET,seqET,muVarET,betaVarET,
		deltaVarET,equalET,deltaET, veeET,wedgeET,muET, betaET,epsET, kleeneET,identET,assignET;
};
}

namespace etvo
{

  void parser::runCalculatorEtvo()
  {
	string prompt = "->";
    typedef std::string::const_iterator iterator_type;
    typedef calculatorEtvo::calculator<iterator_type> calculator;

    boost::spirit::ascii::space_type space; // Our skipper
	
    calculator calc; // Our grammar
    string line;
    string header = "(Event+Time).Variant Operators - ETVO 2.2\n\n";
    header+="Calculator for Event-variant/Time-variant series \n";
    header += "\t B.Cottenceau L.Hardouin J.Trunk - LARIS - June 2021";
    cout << header << endl;

    cout << endl <<prompt;
    while (getline(cin, line))
    {
	  if (line == "quit") return;
	  if (line == "cls")
	  {
		  system("cls");
		  cout << header << endl;		  
		  line = "";
	  }
	  	 	  
      std::string::const_iterator iter = line.begin();
      std::string::const_iterator end = line.end();

      try {
		  if (line != "")
		  {
			  if (line.front() != '%')
			  {
				  bool r = phrase_parse(iter, end, calc, space);
				  if (r && iter == end)
				  {
					  /*
					  cout << "stack Ed=" << calculatorEtvo::stack_Ed.size() << endl;
					  cout << "stack MM=" << calculatorEtvo::stack_MM.size() << endl;
					  cout << "stack Tg=" << calculatorEtvo::stack_Tg.size() << endl;
					  cout << "stack mEd=" << calculatorEtvo::stack_mEd.size() << endl;
					  cout << "stack mMM=" << calculatorEtvo::stack_mMM.size() << endl;
					  cout << "stack mTg=" << calculatorEtvo::stack_mTg.size() << endl;
					  cout << "vect Uint=" << calculatorEtvo::vect_Uint.size() << endl;
					  cout << "vect Int=" << calculatorEtvo::vect_Int.size() << endl;
					  */					
				  }
				  else
				  {
					  cout << "syntax error";
					  while (calculatorEtvo::stack_ET.size()) calculatorEtvo::stack_ET.pop();
					 // while (calculatorEtvo::stack_Ed.size()) calculatorEtvo::stack_Ed.pop();
					 // while (calculatorEtvo::stack_Tg.size()) calculatorEtvo::stack_Tg.pop();
					 // while (calculatorEtvo::stack_MM.size()) calculatorEtvo::stack_MM.pop();
					 // while (calculatorEtvo::stack_mEd.size()) calculatorEtvo::stack_mEd.pop();
					 // while (calculatorEtvo::stack_mTg.size()) calculatorEtvo::stack_mTg.pop();
					 // while (calculatorEtvo::stack_mMM.size()) calculatorEtvo::stack_mMM.pop();
					  calculatorEtvo::vect_Uint.clear();
					  calculatorEtvo::vect_Int.clear();
				  }
				  cout << endl;
			  }
		  }
        
      }
      catch (const etvoException & exEtvo)
      {
		  cout << exEtvo.Message() << endl;
		while (calculatorEtvo::stack_ET.size()) calculatorEtvo::stack_ET.pop();

       /* while (calculatorEtvo::stack_Ed.size()) calculatorEtvo::stack_Ed.pop();
        while (calculatorEtvo::stack_Tg.size()) calculatorEtvo::stack_Tg.pop();
        while (calculatorEtvo::stack_MM.size()) calculatorEtvo::stack_MM.pop();
		while (calculatorEtvo::stack_mEd.size()) calculatorEtvo::stack_mEd.pop();
		while (calculatorEtvo::stack_mTg.size()) calculatorEtvo::stack_mTg.pop();
		while (calculatorEtvo::stack_mMM.size()) calculatorEtvo::stack_mMM.pop();
		*/
        calculatorEtvo::vect_Uint.clear();
        calculatorEtvo::vect_Int.clear();
      }
      catch (const exception & ex)
      {
        cout << ex.what();
      }
	  cout << prompt;
    }
  }
}