#include "parser.h"
#define BOOST_SPIRIT_NO_PREDEFINED_TERMINALS

#include <boost/config/warning_disable.hpp>
#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/qi_int.hpp>
#include <boost/spirit/include/qi_char_class.hpp>
#include <boost/spirit/include/qi_string.hpp>
#include <iostream>
#include <string>
#include <stack>
#include<map>
#include<algorithm>
#include "../etvo/common/etvoException.h"
#include "../test/factory/randGen.h"

using namespace std;
using namespace etvo;

typedef std::map < std::string, etvo::series> symtab_smm;

typedef std::map < std::string, etvo::matrix<etvo::series>> symtab_mmm;


namespace calculatorEtvo
{
  symtab_smm vars_MM;  // vars in MinMaxGD
  symtab_mmm vars_mMM;  // vars in matrix<MinMaxGD>

  std::stack<series> stack_MM;
  std::stack<etvo::matrix<series>> stack_mMM;
  std::stack<std::string> stack_identMM;
  std::stack<std::string> stack_identmMM;

  std::vector<unsigned> vect_Uint;
  std::vector<int> vect_Int;
  std::string strIdent;

  namespace qi = boost::spirit::qi;
  namespace ascii = boost::spirit::ascii;
 
    
  void do_pushC(char c)
  {
	  strIdent += c;
  }
  void do_pushInt(int w)
  {
	  vect_Int.push_back(w);
  }

  void do_pushUint(unsigned w)
  {
	  vect_Uint.push_back(w);
  }

// all semantic actions are in included source files
#include "doMMGD.cpp"
#include "domMMGD.cpp"  
    

 //------------------------------------------------------------------------------------

  ///////////////////////////////////////////////////////////////////////////////
  //  Our calculator grammar
  ///////////////////////////////////////////////////////////////////////////////
  template <typename Iterator>
  struct calculator : qi::grammar<Iterator, ascii::space_type>
  {
    calculator() : calculator::base_type(statement)
    {
      qi::int_type int_;
      qi::uint_type uint_;
      qi::alnum_type alnum_;
      qi::lexeme_type lexeme_;     
	  gammaMM = 'g' >> int_[&do_pushGammaMM];
      deltaMM = 'd' >> int_[&do_pushDeltaMM];
      epsMM = (lexeme_["eps"])[&do_pushEpsMM];
	  	  
      infMM = ("inf(" >> expressionMM >> ',' >> expressionMM >> ')')[&do_infMM];
      fracMM = ("frac(" >> expressionMM >> ',' >> expressionMM >> ')')[&do_fracMM];
      prcausMM = ("pr+(" >> expressionMM >> ')')[&do_prcausMM];
      equalMM = ("?=(">>identMM[&do_pushVarMM] >> "," >> identMM[&do_pushVarMM]>>')')[&do_equalMM];
	  identMM = lexeme_["s" >> +(alnum_[&do_pushC])];
      assignMM = (identMM[&do_pushIdentMM] >> '=' >> expressionMM)[&do_assignMM];
      mm = ("M{" >> expressionMM >> "}");      
      factorMM = polyMM | groupMM | kleeneMM | entrymMM| identMM[&do_pushVarMM]|infMM|fracMM|prcausMM;
      kleeneMM = ('[' >> expressionMM >> "]*")[&do_kleeneMM];
      groupMM = '(' >> expressionMM >> ')';
      termMM = factorMM >> *('.' >> factorMM[&do_otimesMM]);
      expressionMM = termMM >> *('+' >> termMM[&do_oplusMM]);
      polyMM = gammaMM | deltaMM |epsMM;
	  identmMM = lexeme_["S" >> +(alnum_[&do_pushC])];
	  assignmMM = (identmMM[&do_pushIdentmMM] >> '=' >> expressionmMM)[&do_assignmMM];
	  epsMatMM = ("eps(" >> int_[&do_pushInt] >> ',' >> int_[&do_pushInt] >> ')')[&do_pushMatMM];
	  factormMM = epsMatMM | groupmMM | kleenemMM | identmMM[&do_pushVarmMM] | infmMM | lfracmMM | rfracmMM | prcausmMM;
	  kleenemMM = ('[' >> expressionmMM >> "]*")[&do_kleenemMM];
	  groupmMM = '(' >> expressionmMM >> ')';
	  termmMM = factormMM >> *('.' >> factormMM[&do_otimesmMM]);
	  expressionmMM = termmMM >> *('+' >> termmMM[&do_oplusmMM]);
	  assignEntrymMM = (identmMM[&do_pushIdentmMM] >> '(' >> int_[&do_pushUint] >> ',' >> int_[&do_pushUint] >> ")=" >> expressionMM)[&do_assignEntrymMM];
	  entrymMM = (identmMM[&do_pushIdentmMM] >> '(' >> int_[&do_pushUint] >> ',' >> int_[&do_pushUint] >> ")")[&do_pushEntrymMM];
	  infmMM = ("inf(" >> expressionmMM >> ',' >> expressionmMM >> ')')[&do_infmMM];
	  lfracmMM = ("lfrac(" >> expressionmMM >> ',' >> expressionmMM >> ')')[&do_lfracmMM];
	  rfracmMM = ("rfrac(" >> expressionmMM >> ',' >> expressionmMM >> ')')[&do_rfracmMM];
	  prcausmMM = ("pr+(" >> expressionmMM >> ')')[&do_prcausmMM];

	  statement =  equalMM | assignMM		
		  | assignmMM | assignEntrymMM
		  | identMM[&do_printVarMM] 
		  | identmMM[&do_printVarmMM]		  
		  | mm[&do_printMM];
    }

	qi::rule<Iterator, ascii::space_type> infmMM, lfracmMM, rfracmMM, identmMM, assignmMM,
		assignEntrymMM, expressionmMM, epsMatMM, factormMM,
		kleenemMM, groupmMM, termmMM, entrymMM, prcausmMM;
		
    qi::rule<Iterator, ascii::space_type> statement;
    qi::rule<Iterator, ascii::space_type> expressionMM,mm,gammaMM,deltaMM,infMM,fracMM,prcausMM,EdToMM,TgToMM,equalMM,
      factorMM, groupMM, termMM, polyMM, epsMM, kleeneMM, identMM, assignMM;
    };
}

namespace etvo
{

  void parser::runCalculatorEtvo()
  {
	string prompt = "->";
    typedef std::string::const_iterator iterator_type;
    typedef calculatorEtvo::calculator<iterator_type> calculator;

    boost::spirit::ascii::space_type space; // Our skipper
	
    calculator calc; // Our grammar
    string line;
    string header = "MinMax[[g,d]] \n\n";
    header+="Calculator for series in MinMax[[g,d]\n";
    header += "\t - LARIS - June 2021";
    cout << header << endl;

    cout << endl <<prompt;
    while (getline(cin, line))
    {
	  if (line == "quit") return;
	  if (line == "cls")
	  {
		  system("cls");
		  cout << header << endl;		  
		  line = "";
	  }
	  	 	  
      std::string::const_iterator iter = line.begin();
      std::string::const_iterator end = line.end();

      try {
		  if (line != "")
		  {
			  if (line.front() != '%')
			  {
				  bool r = phrase_parse(iter, end, calc, space);
				  if (r && iter == end)
				  {
					  /*
					  cout << "stack Ed=" << calculatorEtvo::stack_Ed.size() << endl;
					  cout << "stack MM=" << calculatorEtvo::stack_MM.size() << endl;
					  cout << "stack Tg=" << calculatorEtvo::stack_Tg.size() << endl;
					  cout << "stack mEd=" << calculatorEtvo::stack_mEd.size() << endl;
					  cout << "stack mMM=" << calculatorEtvo::stack_mMM.size() << endl;
					  cout << "stack mTg=" << calculatorEtvo::stack_mTg.size() << endl;
					  cout << "vect Uint=" << calculatorEtvo::vect_Uint.size() << endl;
					  cout << "vect Int=" << calculatorEtvo::vect_Int.size() << endl;
					  */					
				  }
				  else
				  {
					  cout << "syntax error";
					  while (calculatorEtvo::stack_MM.size()) calculatorEtvo::stack_MM.pop();
					  while (calculatorEtvo::stack_mMM.size()) calculatorEtvo::stack_mMM.pop();
					  calculatorEtvo::vect_Uint.clear();
					  calculatorEtvo::vect_Int.clear();
				  }
				  cout << endl;
			  }
		  }
        
      }
      catch (const etvoException & exEtvo)
      {
        cout << exEtvo.Message();
        while (calculatorEtvo::stack_MM.size()) calculatorEtvo::stack_MM.pop();
		while (calculatorEtvo::stack_mMM.size()) calculatorEtvo::stack_mMM.pop();
        calculatorEtvo::vect_Uint.clear();
        calculatorEtvo::vect_Int.clear();
      }
      catch (const exception & ex)
      {
        cout << ex.what();
      }
	  cout << prompt;
    }
  }
}