#ifndef PARSER_H
#define PARSER_H

#include "../etvo/seriesTg/seriesTg.h"
#include "../etvo/seriesEd/seriesEd.h"
#include "../etvo/seriesET/seriesET.h"

//#include "../etvo/wrapperMMGD/polyWrapper.h"


/*!
 * \class etvo::parser
 *
 * \brief Class methods to parse strings describing formal series
 *
 *
 * \author BC LH JT LARIS
 * \version 2.0
 */
#include<string>
namespace etvo{
class parser
{
    public:
		/// parser for MinMaxGD polynomials. A syntax error throws an exception. 
		/// s="(g1.d2+g3.d4+g5.d7).(g2.d4+g1.d2)"
		static poly parsePoly(const std::string & str);
		/// parser for balanced E[[d]] polynomials. A syntax error throws an exception. 
		/// A polynomial must be balanced, all terms must have the same gain
		/// s="(g1.m2.b2.d2+g3.d4+g5.m3.b3.g2.d7).(m3.b2.g1.d2)"
		static polyEd parsePolyEd(const std::string & s);
		/// parser for balanced T[[g]] polynomials. A syntax error throws an exception.  
		/// A polynomial must be balanced, all terms must have the same gain
		/// s="(d1.v2.w2.g2+d4.g3+d7.v3.w3.d1.g5).(v2.w3.g2+d4.v2.w3.g3)"
		static polyTg parsePolyTg(const std::string & s);
		/// parser for balanced series in E[[d]]. A syntax error throws an exception. 
		///  s="g1.d2 + [g2.m3.b3.g1.d3 + g4.d4]*"
		static seriesEd parseSeriesEd(const std::string & s);
		/// parser for balanced series in T[[g]]. A syntax error throws an exception. 
		/// s="g1.d2 + [d2.v3.w3.d1.g3 + d4.g4]*"
		static seriesTg parseSeriesTg(const std::string & s);


        static void runCalculatorEtvo();

		static series parseSeries(const std::string & s);


};

}

#endif // PARSER_H
