#include "parser.h"
#define BOOST_SPIRIT_NO_PREDEFINED_TERMINALS
#include <boost/config/warning_disable.hpp>
#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/qi_int.hpp>
#include <iostream>
#include <string>
#include <stack>
#include "../etvo/common/etvoException.h"

using namespace std;
using namespace etvo;

namespace parseseriesIII
{
  std::stack<etvo::series> pSEdStack;
  std::vector<unsigned> vect_Uint;

  namespace qi = boost::spirit::qi;
  namespace ascii = boost::spirit::ascii;

  ///////////////////////////////////////////////////////////////////////////////
  //  Semantic actions
  ////////////////////////////////////////////////////////1///////////////////////
  
  void do_print() {
    /*if(pEdStack.size()!=0)
    {
    polyEd last=pEdStack.top();
    pEdStack.top();
    cout << last<<endl;
    }*/
  }
  void do_kleene() 
  {
    series last = pSEdStack.top();
    pSEdStack.pop();
    last = last.star();
    pSEdStack.push(last);
  }

  void do_pushGamma(int n) {
    series p(gd(n,0));
    pSEdStack.push(p);
  }
  void do_pushDelta(int t) {
    series p(gd(0,t));
    pSEdStack.push(p);
  }  

  void do_pushUint(unsigned w)
  {
	  vect_Uint.push_back(w);
  }
  
  void do_otimes() {
    series last = pSEdStack.top();
    pSEdStack.pop();
    series first = pSEdStack.top();
    pSEdStack.pop();
    series res = first * last;
    pSEdStack.push(res);
  }

  void do_oplus() {
    series last = pSEdStack.top();
    pSEdStack.pop();
    series first = pSEdStack.top();
    pSEdStack.pop();
    series res = first + last;
    pSEdStack.push(res);
  }

  

  ///////////////////////////////////////////////////////////////////////////////
  //  Our calculator grammar
  ///////////////////////////////////////////////////////////////////////////////
  template <typename Iterator>
  struct calculator : qi::grammar<Iterator, ascii::space_type>
  {
    calculator() : calculator::base_type(statement)
    {
      qi::int_type int_;
	  qi::uint_type uint_;

      statement = (expression[&do_print]);
      factor = poly | group| kleene;
      kleene = ('[' >>expression >> "]*")[&do_kleene];
      group = '(' >> expression >> ')';
      term = factor >> *('.' >> factor[&do_otimes]);
      expression = term >> *('+' >> term[&do_oplus]);
      poly = gamma | delta ;
      gamma = 'g' >> int_[&do_pushGamma];
      delta = 'd' >> int_[&do_pushDelta];      
    }

    qi::rule<Iterator, ascii::space_type> statement, expression,
      factor, group, term, poly, nabla, gamma, delta, mu, beta,kleene,seqEd,muVarEd,betaVarEd;
  };
}

namespace etvo
{
  series parser::parseSeries(const std::string & str)
  {
    typedef std::string::const_iterator iterator_type;
    typedef parseseriesIII::calculator<iterator_type> calculator;

    boost::spirit::ascii::space_type space; // Our skipper
    calculator calc; // Our grammar

    std::string::const_iterator iter = str.begin();
    std::string::const_iterator end = str.end();
    bool r = phrase_parse(iter, end, calc, space);

    if (r && iter == end)
    {
      series result = parseseriesIII::pSEdStack.top();
      parseseriesIII::pSEdStack.pop();
      return result;
    }
    else
    {
      throw etvoException(14, "parser failed");
    }
  }
}